/*
 *  combatant.cc is part of Freebooters
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 */

#include "combatant.h"

Combatant::Combatant(Player *c, bool is_right):
  firepower(0),
  deathcounter(0),
  lastshot_time(0),
  explosion_frame(33),
  player(c),
  right(is_right)
{
   Tools t;
   explode_sound=t.load_wav("explode.wav");
   shot=t.load_wav("shot.wav");
}

void Combatant::setcolor(){
  ship->setcolbyname(player->get_col());  
}

void Combatant::draw_firepower(SDL_Surface* screen){
  if (firepower==0) return;
  SDL_Rect rect;
  if (firepower<46) {
    rect.y=470-(firepower*10);
    rect.h=firepower*10;
  } else {
    rect.y=10;
    rect.h=460;
  }
  if (right) {
    rect.x=620;
  } else {
    rect.x=10 ;
  }
  rect.w=10;
  int r,g,b;
  ship->getcolrgb(player->get_col(),&r,&g,&b);
  SDL_FillRect(screen,&rect,SDL_MapRGB(screen->format,r,g,b));
}

void Combatant::explode(SDL_Surface* screen){
  // explode twice before sinking

   Tools t;
   if (deathcounter==33 || deathcounter==66) explosion_frame=1;
   if (explosion_frame>32) return;
   if (explosion_frame==1)
      t.play_sound(explode_sound);
   
   SDL_Rect srcrect;
   SDL_Rect dstrect;
   dstrect=ship->get_rect();
   srcrect.y=0;
   srcrect.h=48;
   srcrect.w=48;
   dstrect.h=48;
   dstrect.w=48;
   srcrect.x=explosion_frame*48;
   SDL_BlitSurface(explosion,&srcrect,screen,&dstrect);
   explosion_frame++;
}

void Combatant::unload(){
  Mix_FreeChunk(shot);
  Mix_FreeChunk(explode_sound);
  SDL_FreeSurface(explosion);
  delete ship;
}









