/*
 *  gamemenus.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "menu.h"
#include "gamemenus.h"
#include "player.h"
#include <vector>
#include <string>
#include <sstream>
#include "ship.h"
#include "seabattle.h"

using namespace std;

Gamemenus::Gamemenus()
{
}

Gamemenus::Gamemenus(Properties* p)
{
   properties = p;
}

SDL_Surface* Gamemenus::get_screen()
{
   return properties->screen;
}

void Gamemenus::men_fight_prepare(Player* player, Player* opponent)
{
   Menu menu(properties, 250, 400, 50, 1);
   menu.add_label("Prepare to fight", 170, 80);

   string hn;
   
   if (player->get_ship()->get_flag() == "pi")
      menu.add_image(t.sprites_path("flag-pirate.png"), 40, 180);
   else
   {
      hn = player->get_nationality();
      if (hn == "du")
         menu.add_image(t.sprites_path("flag-dutch.png"), 40, 180);
      if (hn == "en")
         menu.add_image(t.sprites_path("flag-english.png"), 40, 180);
      if (hn == "fr")
         menu.add_image(t.sprites_path("flag-french.png"), 40, 180);
      if (hn == "sp")
         menu.add_image(t.sprites_path("flag-spanish.png"), 40, 180);
   }

   menu.add_label(player->get_name(), 20, 240);
   menu.add_label(player->get_ship()->get_name(), 20, 280);
   menu.add_label(player->get_ship()->get_type_descr(), 20, 320);

   hn = opponent->get_nationality();
   if (hn == "du")
      menu.add_image(t.sprites_path("flag-dutch.png"), 320, 180);
   if (hn == "en")
      menu.add_image(t.sprites_path("flag-english.png"), 320, 180);
   if (hn == "fr")
      menu.add_image(t.sprites_path("flag-french.png"), 320, 180);
   if (hn == "sp")
      menu.add_image(t.sprites_path("flag-spanish.png"), 320, 180);
   if (hn == "pi")
      menu.add_image(t.sprites_path("flag-pirate.png"), 320, 180);

   menu.add_label(opponent->get_name(), 300, 240);
   menu.add_label(opponent->get_ship()->get_name(), 300, 280);
   menu.add_label(opponent->get_ship()->get_type_descr(), 300, 320);

   menu.add_select("Fight!");

   menuselect rt = menu.display();
}

menuselect Gamemenus::men_start()
{
   Menu menu(properties, 240, 185, 50, 3);
   menu.add_select("New game");
   menu.add_select("Load game");
   menu.add_select("Quit game");

   menuselect rt = menu.display();

   return rt;
}

menuselect Gamemenus::men_new_player(harbourlist* harbours)
{
   vector<string> diff;
   diff.push_back("Boring");
   diff.push_back("Challenging");
   diff.push_back("Impossible");

   vector<string> nation;
   nation.push_back("French");
   nation.push_back("Dutch");
   nation.push_back("Spanish");
   nation.push_back("English");

   vector<string> advantage;
   advantage.push_back("Navigation");
   advantage.push_back("Gunning");
   advantage.push_back("Charme");
   advantage.push_back("Medicine");

   vector<string> colors;
   colors.push_back("black");
   colors.push_back("red");
   colors.push_back("blue");
   colors.push_back("pink");
   colors.push_back("green");
   colors.push_back("white");
   colors.push_back("yellow");
   colors.push_back("grey");
   colors.push_back("cyan");
   colors.push_back("brown");

   vector<string> startharbour;
   for (harbourlist::iterator hi = harbours->begin(); hi != harbours->end(); hi++)
      if (hi->has_dock())
         startharbour.push_back(hi->get_name());

   int x = 125;

   Menu menu(properties, 355, 160, 40, 7);
   menu.add_label("Player name", x, 160);
   menu.add_edit_string("Player name", "Largo", false);
   menu.add_label("Difficulty", x, 200);
   menu.add_switch("Difficulty", diff);
   menu.add_label("Nationality", x, 240);
   menu.add_switch("Nationality", nation);
   menu.add_label("Start Harbour", x, 280);
   menu.add_switch("Start Harbour", startharbour);
   menu.add_label("Advantage", x, 320);
   menu.add_switch("Advantage", advantage);
   menu.add_label("Color", x, 360);
   menu.add_switch("Color", colors);
   menu.add_select("Go");

   menuselect rt = menu.display();

   return rt;
}

string Gamemenus::men_harbour(Player* player, World* world)
{
   set_background(player->get_harbour_pos()->get_background());
   Menu menu(properties, 9);

   string hn = player->get_harbour_pos()->get_nation();

   if (hn == "du")
      menu.add_image(t.sprites_path("flag-dutch.png"), menu.get_x(), 20);
   if (hn == "en")
      menu.add_image(t.sprites_path("flag-english.png"), menu.get_x(), 20);
   if (hn == "fr")
      menu.add_image(t.sprites_path("flag-french.png"), menu.get_x(), 20);
   if (hn == "sp")
      menu.add_image(t.sprites_path("flag-spanish.png"), menu.get_x(), 20);

   menu.add_label(player->get_harbour_pos()->get_name(), menu.get_x()+60, 20);

   menu.add_select("Visit the market");

   if (player->get_harbour_pos()->has_governor())
      menu.add_select("Visit the gouvernor");

   if (player->get_harbour_pos()->has_dock())
      menu.add_select("Visit the shipyard");

   menu.add_select("Visit the tavern");
   menu.add_select("View status information");

   if (player->has_ship())
      menu.add_select("Edit ship settings");

   menu.add_select("Game options");
   menu.add_select("Sail away");
   menu.add_select("End turn");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

void Gamemenus::display_statusbar(Player* player, Menu* menu)
{
   ostringstream foodtitle;
   foodtitle << (int) player->get_ship()->get_food() << "/" << player->get_ship()->get_food_capacity();

   ostringstream cargo;
   cargo << player->get_ship()->get_cargo() << "/" << player->get_ship()->get_cargo_capacity();

   menu->add_label("Day", 20, 20);
   menu->add_label(player->get_round(), 20, 55);
   menu->add_image_trans(t.sprites_path("chest.png"), 10, 115);
   menu->add_label(player->get_gold(), 20, 140);
   menu->add_image_trans(t.sprites_path("barrel.png"), 10, 222);
   menu->add_label(cargo.str(), 20, 240);
   menu->add_image_trans(t.sprites_path("bottle.png"), 4, 315);
//   menu->add_label("Food", 20, 300);
   menu->add_label(foodtitle.str(), 20, 340);
   menu->add_image_trans(t.sprites_path("hook.png"), 4, 395);
   menu->add_label(player->get_ship()->get_crew_members(), 20, 420);
}

void Gamemenus::display_statusbar(Player* player, Menu* menu, string title)
{
   menu->add_label(title, 130, 20);
   display_statusbar(player, menu);
}

void Gamemenus::display_statusbar(Player* player, Menu* menu, string title, string title2)
{
   menu->add_label(title, 130, 20);
   menu->add_label(title2, 130, 50);
   display_statusbar(player, menu);
}

void Gamemenus::display_statusbar(Menu* menu, string title, string title2)
{
   menu->add_label(title, 130, 20);
   menu->add_label(title2, 130, 50);
}

// This menu doesn't have the sidebar
string Gamemenus::men_game_options(World* world)
{
   Menu menu(properties, 4);
   menu.add_select("Save game");
   menu.add_select("Load game");
   menu.add_select("Quit game");
   menu.add_select("Back");

   menuselect rt = menu.display();
   return rt.selected;
}


string Gamemenus::men_market_prices(Player* player)
{
   Menu menu(properties, 130, 300, 40, 4);

   int y = 100;

   vector<Good> goods = player->get_harbour_pos()->get_market()->get_goods();
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
   {
      if (gi->get_name() != "Food")
      {
         menu.add_label(gi->get_name(), 130, y);
         menu.add_label(gi->get_price(), 300, y);
         menu.add_label(gi->get_stock(), 400, y);
         y += 40;
      }
   }

   menu.add_select("Back");

   y = 100;
   vector<Good> ogoods = player->get_ship()->get_goods();
   for (vector<Good>::iterator gi = ogoods.begin(); gi != ogoods.end(); gi++)
   {
      if (gi->get_name() != "Food")
      {
         menu.add_label(gi->get_stock(), 520, y);
         y += 40;
      }
   }

   display_statusbar(player, &menu, "Good/Price/Available/On ship");

   menuselect rt = menu.display();
   return rt.selected;
}


string Gamemenus::men_market(Player* player)
{
   Menu menu(properties, 5);

   menu.add_select("Buy goods");
   menu.add_select("Sell goods");
   menu.add_select("Prices");
   menu.add_select("Refresh provisions");
   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

std::string Gamemenus::men_ship_rename(Player* player)
{
   Menu menu(properties, 1);
   menu.add_edit_string("Shipname", "", true);

   display_statusbar(player, &menu, "What shall be the vessel's name?");

   menuselect rt = menu.display();
   return rt.strs["Shipname"];
}

int Gamemenus::men_sell_menu(Player* player, string goodname)
{
   Menu menu(properties, 1);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "Sell how much " + goodname + "?");

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

int Gamemenus::men_buy_menu(Player* player, string goodname)
{
   Menu menu(properties, 1);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "Buy how much " + goodname + "?");

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

int Gamemenus::men_buy_food(Player* player)
{
   Menu menu(properties, 1);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "Buy how much food?");

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

// adapts dynamically towards the stock
string Gamemenus::men_select_goods(Player* player, goodslist goods)
{
   Menu menu(properties, 10);

   int y = menu.get_y();
   for (goodslist::iterator gi = goods.begin(); gi != goods.end(); gi++)
   {
      if (gi->get_name() != "Food")
      {
         menu.add_select(gi->get_name());
         menu.add_label(gi->get_stock(), 280, y);
      }
      y += menu.get_h();
   }

   display_statusbar(player, &menu, "Select a good");

   menuselect rt = menu.display();
   return rt.selected;
}

void Gamemenus::msg_box(Player* player, string msg, string backgroundimage)
{
   string backup = get_background();
   set_background(backgroundimage);
   Menu menu(properties, 1);
   set_background(backup);
   menu.add_select("OK");

   display_statusbar(player, &menu, msg);

   menuselect rt = menu.display();
}

void Gamemenus::msg_box(Player* player, string msg_part1, string msg_part2, string backgroundimage)
{
   string backup = get_background();
   set_background(backgroundimage);
   Menu menu(properties, 1);
   set_background(backup);
   menu.add_select("OK");

   display_statusbar(player, &menu, msg_part1, msg_part2);

   menuselect rt = menu.display();
}

void Gamemenus::men_stat_player(Player* player)
{
   Menu menu(properties, 130, 400, 40, 1);

   int y = 80;
   int xlabel = 130;
   int xdata = 410;

   menu.add_label("Gold",       xlabel, y);
   menu.add_label(player->get_gold(), xdata, y);
   y+=40;
   menu.add_label("Charme",     xlabel, y);
   menu.add_label(player->get_charme_descr(), xdata, y);
   y+=40;
   menu.add_label("Navigation", xlabel, y);
   menu.add_label(player->get_navigation_descr(), xdata, y);
   y+=40;
   menu.add_label("Gunning",    xlabel, y);
   menu.add_label(player->get_gunner_descr(), xdata, y);
   y+=40;
   menu.add_label("Leadership", xlabel, y);
   menu.add_label(player->get_leadership_descr(), xdata, y);
   y+=40;
   menu.add_label("Ferocity",   xlabel, y);
   menu.add_label(player->get_ferocity_descr(), xdata, y);
   y+=40;
   menu.add_label("Pirate points",   xlabel, y);
   menu.add_label(player->get_points(), xdata, y);
   y+=40;
   menu.add_label("Letter of marque", xlabel, y);
   if (player->is_freebooter())
      menu.add_label("Yes", xdata, y);
   else
      menu.add_label("No", xdata, y);

   menu.add_select("Back");

   ostringstream ost;
   ost << player->get_name();
   
   if (player->get_nationality() == "du")
      ost << " (Dutch)";
   if (player->get_nationality() == "en")
      ost << " (English)";
   if (player->get_nationality() == "fr")
      ost << " (French)";
   if (player->get_nationality() == "sp")
      ost << " (Spanish)";

   display_statusbar(player, &menu, ost.str());

   menuselect rt = menu.display();
}


string Gamemenus::men_tavern(Player* player)
{
   Menu menu(properties, 3);

   ostringstream ost;
   ost << player->get_harbour_pos()->get_crew_available() << " men are available";
   
   if (player->get_ship()->get_type() != "NONE")
      menu.add_select("Hire crew members");
   menu.add_select("Get the latest gossip");
   menu.add_select("Back");

   display_statusbar(player, &menu, ost.str());

   menuselect rt = menu.display();
   return rt.selected;
}

int Gamemenus::men_hire_crew(Player* player)
{
   Menu menu(properties, 1);
   menu.add_edit_number("Amount", 0, true);

   ostringstream wage;
   wage << "The wage is " << player->get_harbour_pos()->get_crew_pay() << " gold per week.";

   menu.add_label(wage.str(), menu.get_x(), 150);

   display_statusbar(player, &menu, "Hire how many men?");

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

void Gamemenus::men_stat_ship(Player* player)
{
   Ship* ship = player->get_ship();

   string shiptitle;

   if (ship->get_type() == "SCHOONER")
      shiptitle = ship->get_name() + " (Schooner)";

   if (ship->get_type() == "BRIG")
     shiptitle = ship->get_name() + " (Brig)";

   if (ship->get_type() == "FRIGATE")
     shiptitle = ship->get_name() + " (Frigate)";

   if (ship->get_type() == "MERCHANT")
     shiptitle = ship->get_name() + " (Merchant vessel)";

   if (ship->get_type() == "GALLEON")
     shiptitle = ship->get_name() + " (Galleon)";

   ostringstream cargotitle, ammotitle, platingtitle, cannontitle, paytitle, foodtitle;

   cargotitle << ship->get_cargo() << "/" << ship->get_cargo_capacity();
   ammotitle << ship->get_ammunition() << "/" << ship->get_ammo_capacity();
   cannontitle << ship->get_cannons() << "/" << ship->get_cannon_capacity();
   platingtitle << ship->get_plating() << "/" << ship->get_plating_capacity();
   int actualpay = (int)(ship->get_crew_pay() * (((float)ship->get_pay_percent()/100)));
   paytitle << actualpay << " (" << ship->get_pay_percent() << "%)";
   foodtitle << ship->get_food() << "/" << ship->get_food_capacity();

   int xlabel = 130;
   int xdata = 450;
   int y=60;

   Menu menu(properties, 130, 420, 40, 1);

   menu.add_label("State (%)",      xlabel, y);
   menu.add_label(ship->get_state(), xdata, y);
   y+=40;
   menu.add_label("Plating",        xlabel, y);
   menu.add_label(platingtitle.str(), xdata, y);
   y+=40;
   menu.add_label("Cargo",          xlabel, y);
   menu.add_label(cargotitle.str(), xdata, y);
   y+=40;
   menu.add_label("Cannons",        xlabel, y);
   menu.add_label(cannontitle.str(), xdata, y);
   y+=40;
   menu.add_label("Crew members",   xlabel, y);
   menu.add_label(ship->get_crew_members(), xdata, y);
   y+=40;
   menu.add_label("Happy crew (%)", xlabel, y);
   menu.add_label(ship->get_crew_content(), xdata, y);
   y+=40;
   menu.add_label("Weekly pay",     xlabel, y);
   menu.add_label(paytitle.str(), xdata, y);
   y+=40;
   menu.add_label("Set flag",       xlabel, y);

   if (ship->get_flag() == "pi")
      menu.add_label("Jolly Roger", xdata, y);
   else
      menu.add_label("Royal", xdata, y);
   y+=40;
   menu.add_label("Provisions",     xlabel, y);
   menu.add_label(foodtitle.str(), xdata, y);

   menu.add_select("Back");

   display_statusbar(player, &menu, shiptitle);

   menuselect rt = menu.display();
}

// Returns the selected harbour, otherwise 0
Harbour* Gamemenus::men_set_sail(Harbour* current, World* world)
{
   harbourlist* harbours = world->get_harbours();

   string backup = get_background();
   set_background("caribbean.jpg");
   Menu menu(properties, 10, 10, 10, 10);
   set_background(backup);

   x11pos_t x11pos;

   for (harbourlist::iterator hi = harbours->begin(); hi != harbours->end(); hi++)
   {
      // One arc minute at the equator is one nautical mile
      ostringstream ost;
      ost << hi->get_name() << " (" << (int) world->compute_harbour_distance(current, &(*hi)) << ")";
      x11pos = hi->get_x11_position();

      if (x11pos.x > 500)
         menu.add_select_static(ost.str(), x11pos.x-25, x11pos.y+25, true);
      else
         menu.add_select_static(ost.str(), x11pos.x, x11pos.y+25, true);
      string hn = hi->get_nation();
      
      if (hn == "du")
         menu.add_image(t.sprites_path("flag-dutch-small.png"), x11pos.x, x11pos.y);
      if (hn == "en")
         menu.add_image(t.sprites_path("flag-english-small.png"), x11pos.x, x11pos.y);
      if (hn == "fr")
         menu.add_image(t.sprites_path("flag-french-small.png"), x11pos.x, x11pos.y);
      if (hn == "sp")
         menu.add_image(t.sprites_path("flag-spanish-small.png"), x11pos.x, x11pos.y);

   }
   
   menuselect rt = menu.display_static(true);

   string basicname;
   basicname = rt.selected;
   basicname.erase(basicname.find_first_of("("), basicname.find_first_of(")"));
   basicname.erase(basicname.find_last_of(" "), basicname.find_last_of(" "));

   for (harbourlist::iterator hi = harbours->begin(); hi != harbours->end(); hi++)
   {
      if ( hi->get_name() == basicname)
      {
         if (&(*hi) != current)
            return &(*hi);
         else
            return 0;
      }
   }
   
   return 0;
}

string Gamemenus::men_ship_encounter(Player* player, Player* opponent)
{
   string ship_graphic;

   string nation = opponent->get_nationality();

   if (nation == "pi")
      ship_graphic = "pirates.jpg";

   if (nation == "en")
      ship_graphic = "britain.jpg";

   if (nation == "du")
      ship_graphic = "dutch.jpg";

   if (nation == "fr")
      ship_graphic = "france.jpg";

   if (nation == "sp")
      ship_graphic = "spain.jpg";

   string backup = get_background();
   set_background(ship_graphic);
   Menu menu(properties, 255, 290, 60, 3);
   set_background(backup);

   menu.add_label("A ship has been sighted!", 255, 230);
   menu.add_select("Attack");
   menu.add_select("Sail away");
//   menu.add_select("Try to go on board");

   menuselect rt = menu.display();
   return rt.selected;
}

void Gamemenus::men_stat_cargo(Player* player)
{
   Menu menu(properties, 130, 350, 60, 1);

   int y = 90;
   vector<Good> goods = player->get_ship()->get_goods();
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
   {
      menu.add_label(gi->get_name(), menu.get_x(), y);
      menu.add_label(gi->get_stock(), 360, y);
      y += 35;
   }

   ostringstream ost;
   ost << player->get_ship()->get_cargo()  << "/" << player->get_ship()->get_cargo_capacity();
   
   menu.add_label("Cargo", menu.get_x(), y);
   menu.add_label(ost.str(), 360, y);
   y += 35;

   ostringstream foodtitle;
   foodtitle << player->get_ship()->get_food() << "/" << player->get_ship()->get_food_capacity();
   menu.add_label("Food", menu.get_x(), y);
   menu.add_label(foodtitle.str(), 360, y);
   menu.add_select("Back");

   display_statusbar(player, &menu, "Cargo on " + player->get_ship()->get_name());

   menuselect rt = menu.display();

}

string Gamemenus::men_stat_select(Player* player)
{
   Menu menu(properties, 4);

   menu.add_select("View player status");
   if (player->has_ship())
      menu.add_select("View ship status");
   menu.add_select("View cargo status");
   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

string Gamemenus::men_dock(Player* player)
{
   Menu menu(properties, 5);

   if (!player->has_ship())
   {
      menu.add_select("Buy a ship");
      menu.add_select("Back");
   }
   else
   {
      menu.add_select("Improve equipment");
      menu.add_select("Repair ship");
      menu.add_select("Upgrade ship");
      menu.add_select("Back");
   }

   display_statusbar(player, &menu);
   
   menuselect rt = menu.display();
   return rt.selected;
}

string Gamemenus::men_dock_upgrade_equipment(Player* player)
{
   Menu menu(properties, 3);

   menu.add_select("Buy more cannons");
   menu.add_select("Improve plating");
   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

// returns the amount of bought cannons
int Gamemenus::men_dock_upgrade_cannons(Player* player)
{
   ostringstream capa;
   menuselect rt;

   int maxcannons = (player->get_ship()->get_cannon_capacity()-player->get_ship()->get_cannons());
   capa << "You may install up to " << maxcannons << " cannons.";

   Menu menu(properties, 2);
   menu.add_label(capa.str(), 130, 150);
   menu.add_label("Each cannons costs 40 gold", 130, 210);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "Buy how many cannons?");

   while (rt.selected != "Done")
      rt = menu.display();
   
   return rt.ints["Amount"];
}

int Gamemenus::men_dock_upgrade_plating(Player* player)
{
   ostringstream ost;
   menuselect rt;
   
   int maxplating = (player->get_ship()->get_plating_capacity()-player->get_ship()->get_plating());
   ost << "You may buy " << maxplating << " plating levels";

   Menu menu(properties, 2);
   menu.add_label("Each plating level costs 50 gold", 130, 150);
   menu.add_label(ost.str(), 130, 210);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "Buy how many plating levels?");

   while (rt.selected != "Done")
      rt = menu.display();
   
   return rt.ints["Amount"];
}

// return the repaired percentage
int Gamemenus::men_dock_repair(Player* player)
{
   ostringstream ost, ostprice;
   menuselect rt;
   
   ostprice << "Each repaired % costs "
            << player->get_harbour_pos()->get_ship_price(player->get_ship()->get_type())/100 << " gold";

   int maxrepair = 100-(player->get_ship()->get_state());
   ost << "You may repair up to " << maxrepair << "%";

   Menu menu(properties, 2);
   menu.add_label(ostprice.str(), 130, 150);
   menu.add_label(ost.str(), 130, 210);
   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, "How much to repair?");

   while (rt.selected != "Done")
      rt = menu.display();
   
   return rt.ints["Amount"];
}

string Gamemenus::men_dock_buy_ship(Player* player)
{
   ostringstream ost;
   menuselect rt;

   Menu menu(properties, 6);

   menu.add_select("Merchant vessel");
   menu.add_label(player->get_harbour_pos()->get_ship_price("MERCHANT"), 420, 100);
   menu.add_select("Galleon");
   menu.add_label(player->get_harbour_pos()->get_ship_price("GALLEON"), 420, 150);
   menu.add_select("Brig");
   menu.add_label(player->get_harbour_pos()->get_ship_price("BRIG"), 420, 200);
   menu.add_select("Schooner");
   menu.add_label(player->get_harbour_pos()->get_ship_price("SCHOONER"), 420, 250);
   menu.add_select("Frigate");
   menu.add_label(player->get_harbour_pos()->get_ship_price("FRIGATE"), 420, 300);
   menu.add_select("Back");

   display_statusbar(player, &menu, "Which vessel do you want?");

   while (rt.selected == "void")
      rt = menu.display();

   if (rt.selected == "Merchant vessel")
      return "MERCHANT";
   if (rt.selected == "Galleon")
      return "GALLEON";
   if (rt.selected == "Brig")
      return "BRIG";
   if (rt.selected == "Schooner")
      return "SCHOONER";
   if (rt.selected == "Frigate")
      return "FRIGATE";

   return "NONE";
}

string Gamemenus::men_stat_nations(Player* player, World* world)
{
   nationlist* nations = world->get_nations();

   Menu menu(properties, 130, 300, 35, 1);

   int y = 100;
   for (nationlist::iterator ni = nations->begin(); ni != nations->end(); ni++)
   {
      menu.add_label(ni->get_name(), 130, y);
      menu.add_label(ni->get_rank_descr(player), 360, y);
      y += 50;
   }

   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

string Gamemenus::men_settings_ship(Player* player)
{
   Menu menu(properties, 4);

   menu.add_select("Adjust crew pay");
   menu.add_select("Rename ship");
   menu.add_select("Change the flag");
   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

int Gamemenus::men_settings_crewpay(Player* player)
{
   Menu menu(properties, 2);
   menu.add_edit_number("Amount", 100, true);

   display_statusbar(player, &menu, "Pay how many % of the wages?");

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

bool Gamemenus::bool_box(Player* player, string msg, string backgroundimage)
{
   string backup = get_background();
   set_background(backgroundimage);
   Menu menu(properties, 2);
   set_background(backup);
   menu.add_select("Yes");
   menu.add_select("No");

   display_statusbar(player, &menu, msg);

   menuselect rt = menu.display();

   if (rt.selected == "Yes")
      return true;

   return false;
}

string Gamemenus::men_onsea(Player* player, World* world)
{
   Menu menu(properties, 4);

   string hn = player->get_harbour_pos()->get_nation();

   if (player->get_ship()->get_flag() == "pi")
      menu.add_image(t.sprites_path("flag-pirate.png"), 580, 26);
   else
   {
      if (hn == "du")
         menu.add_image(t.sprites_path("flag-dutch.png"), 580, 26);
      if (hn == "en")
         menu.add_image(t.sprites_path("flag-english.png"), 580, 26);
      if (hn == "fr")
         menu.add_image(t.sprites_path("flag-french.png"), 580, 26);
      if (hn == "sp")
         menu.add_image(t.sprites_path("flag-spanish.png"), 580, 26);
   }

   ostringstream target;
   target << player->get_target_harbour()->get_name() << " " << (int) player->get_remaining_distance() << " (nm)";

   menu.add_select("View status information");
   menu.add_select("Edit ship settings");
   menu.add_select("Game options");
   menu.add_select("End turn");
   menu.set_active(3);

   display_statusbar(player, &menu, target.str());

   menuselect rt = menu.display();
   return rt.selected;
}

string Gamemenus::men_fight_won(Player* player, Player* opponent)
{
   Menu menu(properties, 3);

   ostringstream ost, ost2;
   ost << "You receive " << opponent->get_gold() << " gold.";
   ost2 << "You receive " << opponent->get_ship()->get_food() << " provisions.";
   
   menu.add_select("Loot the ship");
   menu.add_select("Scuttle the ship");
   menu.add_select("Let them sail away");

   display_statusbar(player, &menu, ost.str(), ost2.str());

   menuselect rt = menu.display();
   return rt.selected;
}

int Gamemenus::men_fight_loot(Player* player, string goodname)
{
   Menu menu(properties, 1);

   ostringstream ost, ost2;
   ost << "How much " << goodname << " shall we";
   ost2 << "loot? (max. " << player->get_ship()->get_cargo_free() << ")";

   menu.add_edit_number("Amount", 0, true);

   display_statusbar(player, &menu, ost.str(), ost2.str());

   menuselect rt = menu.display();
   return rt.ints["Amount"];
}

void Gamemenus::set_background(std::string new_background)
{
   properties->background = new_background;
}

std::string Gamemenus::get_background()
{
   return properties->background;
}

string Gamemenus::men_mutiny_select(Player* player)
{
   Menu menu(properties, 3);
   menu.add_select("Careen the leaders");
   menu.add_select("Bribe them");
   menu.add_select("Do nothing");

   display_statusbar(player, &menu, "A mutiny broke out");

   menuselect rt = menu.display();
   return rt.selected;
}

string Gamemenus::men_gov_select(Player* player)
{
   Menu menu(properties, 2);
   menu.add_select("Negotiate a letter of marque");
   menu.add_select("Back");

   display_statusbar(player, &menu);

   menuselect rt = menu.display();
   return rt.selected;
}

void Gamemenus::men_player_dead(Player* player)
{
   set_background("graves.jpg");
   Menu menu(properties, 120, 400, 40, 1);
   menu.add_label(player->get_name() + " has passed away", 120, 80);
   menu.add_label("Gold",       120, 120);
   menu.add_label("Charme",     120, 160);
   menu.add_label("Navigation", 120, 200);
   menu.add_label("Gunning",    120, 240);
   menu.add_label("Leadership", 120, 280);
   menu.add_label("Ferocity",   120, 320);
   menu.add_label("Pirate points",   120, 360);

   menu.add_label(player->get_gold(), 320, 120);
   menu.add_label(player->get_charme_descr(), 320, 160);
   menu.add_label(player->get_navigation_descr(), 320, 200);
   menu.add_label(player->get_gunner_descr(), 320, 240);
   menu.add_label(player->get_leadership_descr(), 320, 280);
   menu.add_label(player->get_ferocity_descr(), 320, 320);
   menu.add_label(player->get_points(), 320, 360);

   menu.add_select("Back");

   menuselect rt = menu.display();
}

void Gamemenus::men_fight_board(Player* player, Player* opponent, bool success)
{
   Menu menu(properties, 280, 440, 40, 1);
   menu.add_label("The ship is boarded", 170, 80);

   string hn;

   if (player->get_ship()->get_flag() == "pi")
      menu.add_image(t.sprites_path("flag-pirate.png"), 40, 180);
   else
   {
      hn = player->get_nationality();
      if (hn == "du")
         menu.add_image(t.sprites_path("flag-dutch.png"), 40, 180);
      if (hn == "en")
         menu.add_image(t.sprites_path("flag-english.png"), 40, 180);
      if (hn == "fr")
         menu.add_image(t.sprites_path("flag-french.png"), 40, 180);
      if (hn == "sp")
         menu.add_image(t.sprites_path("flag-spanish.png"), 40, 180);
   }
   
   menu.add_label(player->get_name(), 20, 240);
   menu.add_label(player->get_ship()->get_name(), 20, 280);
   menu.add_label(player->get_ship()->get_type_descr(), 20, 320);

   hn = opponent->get_nationality();
   if (hn == "du")
      menu.add_image(t.sprites_path("flag-dutch.png"), 320, 180);
   if (hn == "en")
      menu.add_image(t.sprites_path("flag-english.png"), 320, 180);
   if (hn == "fr")
      menu.add_image(t.sprites_path("flag-french.png"), 320, 180);
   if (hn == "sp")
      menu.add_image(t.sprites_path("flag-spanish.png"), 320, 180);
   if (hn == "pi")
      menu.add_image(t.sprites_path("flag-pirate.png"), 320, 180);

   menu.add_label(opponent->get_name(), 300, 240);
   menu.add_label(opponent->get_ship()->get_name(), 300, 280);
   menu.add_label(opponent->get_ship()->get_type_descr(), 300, 320);

   if (success)
      menu.add_label("You won the gruesome fight", 20, 360);
   else
   {
       menu.add_label("You lost the gruesome fight", 20, 360);
       menu.add_label("All your cargo is lost", 20, 400);
   }

   menu.add_select("Back");

   menuselect rt = menu.display();
}

string Gamemenus::men_opt_savegame(World* world)
{
   vector<savegame_description> names = world->get_savegame_names();

   Menu menu(properties, 9);

   int slots = 0;
   int free_slot = 0;

   for (int i=0; i<6; i++)
   {
      if (names.at(i).name != "")
      {
         menu.add_select(names.at(i).name);
         slots++;
      }

      if (names.at(i).name == "")
         free_slot = i;
   }

   if (slots < 6)
      menu.add_select("Empty slot");

   menu.add_select("Back");

   menuselect rt = menu.display();

   if (rt.selected == "Empty slot")
   {
      ostringstream o;
      o << "save" << free_slot+1 << ".fbs";
      return t.savegame_path(o.str());
   }

   if (rt.selected == names.at(0).name)
      return t.savegame_path("save1.fbs");

   if (rt.selected == names.at(1).name)
      return t.savegame_path("save2.fbs");

   if (rt.selected == names.at(2).name)
      return t.savegame_path("save3.fbs");

   if (rt.selected == names.at(3).name)
      return t.savegame_path("save4.fbs");

   if (rt.selected == names.at(4).name)
      return t.savegame_path("save5.fbs");

   if (rt.selected == names.at(5).name)
      return t.savegame_path("save6.fbs");

   return rt.selected;
}

// returns the filename of the savegame to be loaded
string Gamemenus::men_opt_loadgame(World* world)
{
   // FIXME, the identifiers of the savegame descriptions have to be unique
   // The current system with save slots is ugly, but for a sane solution
   // the menu system needs to be enhanced to support the mouse in dynamic
   // select boxes

   vector<savegame_description> names = world->get_savegame_names();

   Menu menu(properties, 9);

   for (int i=0; i<6; i++)
   {
      if (names.at(i).name != "")
         menu.add_select(names.at(i).name);
   }
   
   menu.add_select("Back");
   menuselect rt = menu.display();

   if (rt.selected == names.at(0).name)
      return t.savegame_path("save1.fbs");

   if (rt.selected == names.at(1).name)
      return t.savegame_path("save2.fbs");

   if (rt.selected == names.at(2).name)
      return t.savegame_path("save3.fbs");

   if (rt.selected == names.at(3).name)
      return t.savegame_path("save4.fbs");

   if (rt.selected == names.at(4).name)
      return t.savegame_path("save5.fbs");

   if (rt.selected == names.at(5).name)
      return t.savegame_path("save6.fbs");
   
   return rt.selected;
}


string Gamemenus::men_opt_savegame_name()
{
   Menu menu(properties, 1);
   menu.add_edit_string("Name", "", true);

   display_statusbar(&menu, "Provide a short description.", "Enter an empty title to abort");

   menuselect rt = menu.display();
   return rt.strs["Name"];
}












