/*
 *  justbattle.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <cstdlib>
#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_image.h>

#include <iostream>
#include <sstream>
#include "menu.h"
#include "world.h"
#include "sdl_setup.h"
#include "player.h"
#include "seabattle.h"
#include "tools.h"

// this is a clone of freebooters.cc, which directly runs a quick battle
// its main function is used in the executable 'buccaneer'

using namespace std;

Tools t;


int main(int argc, char *argv[])
{
  if (argc<2){
    cout << "Usage: buccaneer <Player1> [<Player2>]" << endl;
    cout << "Giving one player name runs a Human vs. AI game" << endl;
    cout << "Giving two player names runs a Human vs. Human game" << endl;
    exit(0);
  }
  
   SDL_Setup setup;
   SDL_Surface* screen = setup.sdl();
   SDL_WM_SetCaption("Buccaneer", 0);
   SDL_Event event;
   Uint8* keys;
   SDL_Surface* title_screen = t.load_img(t.graphics_path("title-buc.jpg"));
   SDL_BlitSurface(title_screen, 0, screen, 0);
   SDL_Flip(screen);
   Mix_Music *titlemusic = t.load_mus("thecatch.xm");
   Mix_PlayMusic(titlemusic, -1 );

   while ( 1 ) {
     while ( SDL_PollEvent(&event) ) {
       switch (event.type) {
       case SDL_QUIT:
	 setup.cleanup();
	 return 0;
       }
     }
     keys = SDL_GetKeyState(0);
     if ( keys[SDLK_ESCAPE] ){
       	 setup.cleanup();
	 return 0;
     }
     if ( keys[SDLK_SPACE] ) {

       ///////////////////////
       // game starts here

       Mix_PauseMusic();
       World world;
       string p1name="Player 1";
       string p2name="Player 2";
       if (argc>1) p1name=argv[1];
       if (argc>2) p2name=argv[2];
       
       world.add_player(Player(p1name, "sp", "DIFF_EASY", world.get_harbour_by_name("Antigua"),"Gunning", t.random_col()));
       world.get_player(0)->set_ship("ST_FRIGATE","Mayflower", 100);
       world.get_player(0)->get_ship()->add_plating(6);
       
       Seabattle* quick;
       
       if (argc>2) {
	 world.add_player(Player(p2name, "sp", "DIFF_EASY", world.get_harbour_by_name("Antigua"),"Gunning", t.random_col()));
	 world.get_player(1)->set_ship("ST_FRIGATE","Mayflower", 100);
	 world.get_player(1)->get_ship()->add_plating(6);
	 quick = new Seabattle(screen,world.get_player(0),world.get_player(1));   
       } else {
	 world.create_opponent();
	 world.get_opponent(0)->set_ship("ST_FRIGATE","Santa Maria", 100);
	 world.get_opponent(0)->get_ship()->add_plating(6);
	 quick = new Seabattle(screen,world.get_player(0),world.get_opponent(0));
       }
       quick->run();
       delete quick;
       //////////////////////
       // game ends here

       SDL_BlitSurface(title_screen, 0, screen, 0);
       SDL_Flip(screen);
       Mix_PlayMusic(titlemusic, -1 );
       SDL_Delay(100);
     }     
     SDL_Delay(100);
   }
}





