/*
 *  market.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string>
#include <vector>
#include "harbour.h"
#include "good.h"
#include "readdata.h"
#include "world.h"

using namespace std;

Market::Market(string harbourname)
{
   readdata hdata(harbourname);

   Good sugar;
   Good rum;
   Good tobacco;
   Good food;
   Good ammo;
   Good weapons;
   
   sugar.set_name("Sugar");
   rum.set_name("Rum");
   tobacco.set_name("Tobacco");
   food.set_name("Food");
   ammo.set_name("Ammo");
   weapons.set_name("Weapons");

   sugar.set_price(hdata.get_int("price_sugar"));
   rum.set_price(hdata.get_int("price_rum"));
   tobacco.set_price(hdata.get_int("price_tobacco"));
   food.set_price(hdata.get_int("price_food"));
   ammo.set_price(hdata.get_int("price_ammo"));
   weapons.set_price(hdata.get_int("price_weapons"));

   sugar.set_demand(hdata.get_int("demand_sugar"));
   rum.set_demand(hdata.get_int("demand_rum"));
   tobacco.set_demand(hdata.get_int("demand_tobacco"));
   food.set_demand(hdata.get_int("demand_food"));
   ammo.set_demand(hdata.get_int("demand_ammo"));
   weapons.set_demand(hdata.get_int("demand_weapons"));

   sugar.set_stock(hdata.get_int("stock_sugar"));
   rum.set_stock(hdata.get_int("stock_rum"));
   tobacco.set_stock(hdata.get_int("stock_tobacco"));
   food.set_stock(hdata.get_int("stock_food"));
   ammo.set_stock(hdata.get_int("stock_ammo"));
   weapons.set_stock(hdata.get_int("stock_weapons"));

   sugar.set_production(hdata.get_int("price_sugar")/4);
   rum.set_production(hdata.get_int("price_rum")/4);
   tobacco.set_production(hdata.get_int("price_tobacco")/4);
   food.set_production(hdata.get_int("price_food")/4);
   ammo.set_production(hdata.get_int("price_ammo")/4);
   weapons.set_production(hdata.get_int("price_weapons")/4);

   goods.push_back(sugar);
   goods.push_back(rum);
   goods.push_back(tobacco);
   goods.push_back(ammo);
   goods.push_back(weapons);
   goods.push_back(food);
}

Market::Market(Savegame* sg)
{
   for (int i=0; i<6; i++)
      goods.push_back(Good(sg));
}


std::vector<Good> Market::get_goods()
{
   return goods;
}

Market::Market()
{
}

int Market::get_stock(string tomatch)
{
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == tomatch)
         return gi->get_stock();

   return 0;
}

int Market::sell(int amount, string goodid, Ship* ship)
{
   int salesvol = 0;
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == goodid)
      {
         gi->add_stock(amount);
         gi->recalculate_price();
         salesvol = gi->get_price()*amount;
         ship->add_good(-amount, goodid);
      }

   return salesvol;
}

int Market::buy(int amount, string goodid, Ship* ship)
{
   int salesvol = 0;
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == goodid)
      {
         gi->add_stock(-amount);
         gi->recalculate_price();
         salesvol = gi->get_price()*amount;
         ship->add_good(amount, goodid);
      }

   return salesvol;
}

int Market::buy_food(int amount, Ship* ship)
{
   int salesvol = 0;
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == "Food")
      {
         gi->add_stock(-amount);
         gi->recalculate_price();
         salesvol = gi->get_price()*amount;
         ship->add_food((float)amount);
      }

   return salesvol;
}
int Market::get_price(string goodid)
{
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == goodid)
         return gi->get_price();

   return 0;
}

void Market::turn(World* world) 
{
   if (world->new_goods_produced())
   {
      for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
         gi->add_stock(gi->get_production());
   }
}

void Market::save_to_file(Savegame* sg)
{
   for (vector<Good>::iterator g = goods.begin(); g != goods.end(); g++)
      g->save_to_file(sg);
}












