/*
 *  menu.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <vector>
#include <string>
#include "menu.h"
#include <sstream>
#include "tools.h"
#include <iostream>
#include <cstring>
#include <unistd.h>

using namespace std;

Properties::Properties(SDL_Color on, SDL_Color off, SDL_Color dark, TTF_Font* fnt, SDL_Surface* scr)
{
   Properties::on_color = on;
   Properties::off_color = off;
   Properties::dark_color = dark;
   Properties::font = fnt;
   Properties::screen = scr;
};

menuselect::menuselect()
{
   selected="void";
};


Properties::Properties()
{

};

Menu::Menu(Properties* p, int xstart, int ystart, int itemheight, int menuitems)
{
   Tools t;
   Menu::properties = *p;
   Menu::currently_editing = false;
   Menu::active = 0;
   Menu::immediate_return = false;
   Menu::x = xstart;
   Menu::y = ystart;
   Menu::height = itemheight;
   Menu::maxitems = menuitems;

   SDL_Surface* temp = t.load_img_alpha(t.graphics_path(p->background));
   Menu::back = SDL_DisplayFormat(temp);

   temp = t.load_img(t.sprites_path("cursor.png"));
   cursorsurface = SDL_DisplayFormat(temp);

   temp = t.load_img_alpha(t.sprites_path("left-arrow.png"));
   SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,20,70,100));
   larrowsurface = SDL_DisplayFormatAlpha(temp);

   temp = t.load_img_alpha(t.sprites_path("right-arrow.png"));
   SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,20,70,100));
   rarrowsurface = SDL_DisplayFormatAlpha(temp);

   SDL_FreeSurface(temp);
}

Menu::Menu(Properties* p, int menuitems)
{
   Tools t;
   Menu::properties = *p;
   Menu::currently_editing = false;
   Menu::active = 0;
   Menu::immediate_return = false;
   Menu::x = 130;
   Menu::y = 100;
   Menu::height = 50;
   Menu::maxitems = menuitems;

   SDL_Surface* temp = t.load_img_alpha(t.graphics_path(p->background));
   Menu::back = SDL_DisplayFormat(temp);

   temp = t.load_img(t.sprites_path("cursor.png"));
   cursorsurface = SDL_DisplayFormat(temp);

   temp = t.load_img_alpha(t.sprites_path("left-arrow.png"));
   SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,20,70,100));
   larrowsurface = SDL_DisplayFormatAlpha(temp);

   temp = t.load_img_alpha(t.sprites_path("right-arrow.png"));
   SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,20,70,100));
   rarrowsurface = SDL_DisplayFormatAlpha(temp);

   SDL_FreeSurface(temp);
}

void Menu::recalculate_pos()
{
   if (items.size() < maxitems)
   {
      for (unsigned int i = 0; i < items.size(); i++)
      {
         SDL_Rect rect;
         rect.x = x;
         rect.y = y + (i*Menu::height);
         rect.w = 0;
         rect.h = 0;
         items[i].item_pos = rect;
      }
   }
   else
   {
      for (unsigned int i = 0; i < items.size(); i++)
      {
         SDL_Rect rect;
         rect.x = x;
         rect.y = y + (i*Menu::height);
         rect.w = 0;
         rect.h = 0;
         items[i].item_pos = rect;
      }
   }

}

int Menu::get_x()
{
   return x;
}

int Menu::get_y()
{
   return y;
}

int Menu::get_h()
{
   return height;
}

void Menu::redraw(bool cursor, bool onoff, bool static_mode, bool small_mode)
{
   Uint32 col = SDL_MapRGB(properties.screen->format, 100, 100, 100);
   SDL_GetRGB(col, properties.screen->format, &properties.dark_color.r, &properties.dark_color.g, &properties.dark_color.b);
   SDL_FillRect(properties.screen, 0, col);
   SDL_BlitSurface(Menu::back, 0, properties.screen, 0);

   recalculate_pos();

   int lower_bound, upper_bound;
   if (items.size() < maxitems)
   {
      lower_bound = 0;
      upper_bound = items.size();
   }
   else
   {
      int upmargin, lowmargin;

      int size = items.size();

      if (maxitems % 2 == 0)
      {
         upmargin = maxitems / 2;
         lowmargin = maxitems / 2;
      }
      else
      {
         upmargin = maxitems / 2;
         lowmargin = (maxitems / 2)+1;
      }

      upper_bound = active + upmargin;
      lower_bound = active - lowmargin;

      if (upper_bound > size)
      {
         lower_bound = size - maxitems;
         upper_bound = size;
      }
      
      if (lower_bound < 0)
      {
         lower_bound = 0;
         upper_bound = 0 + maxitems;
      }

      int c = 0;
      SDL_Rect rect;
      for (int i = lower_bound; i < upper_bound; i++)
      {
         rect.x = x;
         rect.y = c*height+y;
         rect.h = 0;
         rect.w = 0;
         items[i].item_pos = rect;
         c++;
      }
   }

   for (int i = lower_bound; i < upper_bound; i++)
   {
      Tools t;

      SDL_BlitSurface(items[i].off[items[i].subpos], 0, properties.screen, &items[i].item_pos);
      if (items[i].is_switch()) // display two arrows for selecting inside a switch
      {
         SDL_Rect larrow = items[i].item_pos;
         larrow.x -= 23;
         larrow.y += 8;

         SDL_Rect textsize;
         SDL_Rect rarrow;
      
         SDL_GetClipRect(items[i].on[items[i].subpos], &textsize);
         rarrow.x = items[i].item_pos.x + textsize.w + 5;
         rarrow.y = items[i].item_pos.y + 8; 

         SDL_BlitSurface(larrowsurface, 0, properties.screen, &larrow);
         SDL_BlitSurface(rarrowsurface, 0, properties.screen, &rarrow);
      }
   }

   for (unsigned int i = 0; i < static_items.size(); i++)
   {
      SDL_BlitSurface(static_items[i].off[static_items[i].subpos], 0, properties.screen, &static_items[i].item_pos);
   }

   for (unsigned int i = 0; i < static_items_small.size(); i++)
   {
      SDL_BlitSurface(static_items_small[i].off[static_items_small[i].subpos], 0, properties.screen,
                      &static_items_small[i].item_pos);
   }

   for (unsigned int i = 0; i < images.size(); i++)
   {
      SDL_Rect rect = images[i].item_pos;
      SDL_BlitSurface(images[i].img, 0, properties.screen, &images[i].item_pos);
   }

   for (unsigned int i = 0; i < labels.size(); i++)
   {
      SDL_Rect rect = labels[i].item_pos;
      SDL_BlitSurface(labels[i].off[labels[i].subpos], 0, properties.screen, &labels[i].item_pos);
   }

   SDL_Rect textsize;

   if (static_mode)
   {
      SDL_GetClipRect(static_items[active].on[static_items[active].subpos], &textsize);
      SDL_BlitSurface(static_items[active].on[static_items[active].subpos], 0, properties.screen,
                      &static_items[active].item_pos);
      if (static_items[active].text[0].size() == 0)
         textsize.w = 0;
   }
   else
   {
      SDL_GetClipRect(items[active].on[items[active].subpos], &textsize);
      SDL_BlitSurface(items[active].on[items[active].subpos], 0, properties.screen, &items[active].item_pos);
      if (items[active].text.at(0).size() == 0)
         textsize.w = 0;
   }


   if (cursor)
   {
      SDL_Rect cursor;
      cursor.x = items[active].item_pos.x + textsize.w + 5;
      cursor.y = items[active].item_pos.y + 25;  // textsize.h - 5; this sucks when editing blank strings
      cursor.h = 5;
      cursor.w = 5;

      SDL_BlitSurface(cursorsurface, 0, properties.screen, &cursor);
   }

   SDL_Flip(properties.screen);
}

void Menu::add_edit_string(string txt, std::string defaultstr, int x, int y, bool active)
{
   vector<string> tmpvec;
   tmpvec.push_back(defaultstr);
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_EDIT_STR, false);
   if (active)
      currently_editing=true;
   
   if (active)
      immediate_return=true;
   items.push_back(item);
}


void Menu::add_edit_string(string txt, std::string defaultstr, bool active)
{
   vector<string> tmpvec;
   tmpvec.push_back(defaultstr);
   SDL_Rect rect;
   rect.x = 0;   rect.y = 0;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_EDIT_STR, false);
   if (active)
      currently_editing=true;
   
   if (active)
      immediate_return=true;

   items.push_back(item);
}

void Menu::add_edit_password(string txt, std::string defaultstr, bool active)
{
   vector<string> tmpvec;
   tmpvec.push_back(defaultstr);
   SDL_Rect rect;
   rect.x = 0;   rect.y = 0;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_EDIT_STR, false);
   if (active)
      currently_editing=true;
   
   if (active)
      immediate_return=true;
   
   item.set_shadowed();
   items.push_back(item);
}

void Menu::add_slider(string txt, int x, int y, int w, int h, int llim, int ulim)
{
   vector<string> tmpvec;
   tmpvec.push_back(txt);
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = w;   rect.h = h;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_SLIDER, llim, ulim, false);
   items.push_back(item);
}

void Menu::add_image(std::string filename, int x, int y)
{
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Image image(filename, rect, properties, false);
   images.push_back(image);
}

void Menu::add_image_trans(std::string filename, int x, int y)
{
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Image image(filename, rect, properties, true);
   images.push_back(image);
}

void Menu::add_edit_number(string txt, int defaultvalue, int x, int y, bool active)
{
   vector<string> tmpvec;
   ostringstream ost;
   ost << defaultvalue;
   tmpvec.push_back(ost.str());
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_EDIT_NUM, false);
   if (active)
      currently_editing=true;

   if (active)
      immediate_return=true;
   items.push_back(item);
}

void Menu::add_edit_number(string txt, int defaultvalue, bool active)
{
   vector<string> tmpvec;
   ostringstream ost;
   ost << defaultvalue;
   tmpvec.push_back(ost.str());
   SDL_Rect rect;
   rect.x = 0;   rect.y = 0;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_EDIT_NUM, false);
   if (active)
      currently_editing=true;

   if (active)
      immediate_return=true;
   items.push_back(item);
}

void Menu::add_label(string txt, int x, int y)
{
   vector<string> tmpvec;
   tmpvec.push_back(txt);
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_INFO, false);
   labels.push_back(item);
}

void Menu::add_label(int txt, int x, int y)
{
   vector<string> tmpvec;
   ostringstream ost;
   ost << txt;
   tmpvec.push_back(ost.str());
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Item item(ost.str(), tmpvec, rect, properties, MENU_TYPE_INFO, false);
   labels.push_back(item);
}


void Menu::add_select_static(string txt, int x, int y, bool small)
{
   vector<string> tmpvec;
   tmpvec.push_back(txt);
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   if (small)
   {
      Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_SELECT, true);
      static_items.push_back(item);
   }
   else
   {
      Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_SELECT, false);
      static_items.push_back(item);
   }
}

void Menu::add_select(string txt)
{
   vector<string> tmpvec;
   tmpvec.push_back(txt);
   SDL_Rect rect;
   rect.x = 0;   rect.y = 0;   rect.w = 0;   rect.h = 0;

   Menu_Item item(txt, tmpvec, rect, properties, MENU_TYPE_SELECT, false);

   items.push_back(item);
}

void Menu::add_switch(string label, vector<string> txt, int x, int y)
{
   SDL_Rect rect;
   rect.x = x;   rect.y = y;   rect.w = 0;   rect.h = 0;

   Menu_Item item(label, txt, rect, properties, MENU_TYPE_SWITCH, false);
   items.push_back(item);
}

void Menu::add_switch(string label, vector<string> txt)
{
   SDL_Rect rect;
   rect.x = 0;   rect.y = 0;   rect.w = 0;   rect.h = 0;

   Menu_Item item(label, txt, rect, properties, MENU_TYPE_SWITCH, false);
   items.push_back(item);
}

Menu_Image::Menu_Image(std::string filename, SDL_Rect pos, Properties properties, bool transparent)
{
   Tools t;
   Menu_Image::item_pos = pos;

   if (transparent)
   {
      SDL_Surface* temp = t.load_img_alpha(filename);
      SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,20,70,100));
      SDL_SetAlpha(temp, SDL_SRCALPHA, 200);
      Menu_Image::img = SDL_DisplayFormat(temp);
   }
   else
      Menu_Image::img = SDL_DisplayFormat(t.load_img_alpha(filename));
}

Menu_Item::Menu_Item(string mlabel, std::vector<std::string> txt, SDL_Rect pos, Properties prop, menuitem_t mtype, bool small)
{
   Menu_Item::subpos = 0;
   Menu_Item::item_pos = pos;
   Menu_Item::label = mlabel;
   Menu_Item::edit_active = false;
   Menu_Item::shadow_input = false;
   Menu_Item::type = mtype;

   for (unsigned int i=0; i<txt.size(); i++)
   {
      Menu_Item::on.push_back(render_surface_on(txt.at(i).c_str(), &prop, small));
      Menu_Item::off.push_back(render_surface_off(txt.at(i).c_str(), &prop, small));
   }

   Menu_Item::text = txt;
}


Menu_Item::Menu_Item(string mlabel, std::vector<std::string> txt, SDL_Rect pos, Properties prop, menuitem_t mtype,
                     bool active, bool small)
{
   Menu_Item::subpos = 0;
   Menu_Item::item_pos = pos;
   Menu_Item::label = mlabel;
   Menu_Item::edit_active = true;
   Menu_Item::shadow_input = false;
   Menu_Item::type = mtype;

   for (unsigned int i=0; i<txt.size(); i++)
   {
      Menu_Item::on.push_back(render_surface_on(txt.at(i).c_str(), &prop, small));
      Menu_Item::off.push_back(render_surface_off(txt.at(i).c_str(), &prop, small));
   }

   Menu_Item::text = txt;
}


Menu_Item::Menu_Item(string mlabel, std::vector<std::string> txt, SDL_Rect pos, Properties prop,
                     menuitem_t mtype, int llim, int ulim, bool small)
{
   Menu_Item::subpos = 0;
   Menu_Item::item_pos = pos;
   Menu_Item::shadow_input = false;
   Menu_Item::label = mlabel;

   Menu_Item::type = mtype;

   for (unsigned int i=0; i<txt.size(); i++)
   {
      Menu_Item::on.push_back(render_surface_on(txt.at(i).c_str(), &prop, small));
      Menu_Item::off.push_back(render_surface_off(txt.at(i).c_str(), &prop, small));
   }

   Menu_Item::text = txt;
   
}


SDL_Surface* Menu_Item::render_surface_on(string label, Properties* p, bool small)
{
   SDL_Surface* txt;

   if (small)
      txt = TTF_RenderText_Blended(p->small_font, label.c_str(), p->on_color);
   else
      txt = TTF_RenderText_Blended(p->font, label.c_str(), p->on_color);

   return txt;
}

SDL_Surface* Menu_Item::render_surface_off(string label, Properties* p, bool small)
{
   SDL_Surface* txt;
   
   if (small)
      txt = TTF_RenderText_Blended(p->small_font, label.c_str(), p->off_color);
   else
      txt = TTF_RenderText_Blended(p->font, label.c_str(), p->off_color);

   return txt;
}

int Menu_Item::width()
{
   return text.size();
}

void Menu_Item::set_shadowed()
{
   shadow_input = true;
}

int Menu_Item::graphical_width(int pos)
{
   SDL_Rect rect;
   SDL_GetClipRect(on[pos], &rect);
   
   return rect.w;
}

bool Menu_Item::is_editable_str()
{
   if (type == MENU_TYPE_EDIT_STR)
   {
      return true;
   }
   else
      return false;
}

bool Menu_Item::is_editable_int()
{
   if (type == MENU_TYPE_EDIT_NUM)
      return true;
   else
      return false;
}

bool Menu_Item::is_shadowed()
{
   return shadow_input;
}

bool Menu_Item::is_switch()
{
   if (type == MENU_TYPE_SWITCH)
      return true;
   else
      return false;
}

bool Menu_Item::is_editable()
{
   return (is_editable_str() || is_editable_int());
}

bool Menu_Item::is_selectable()
{
   switch (type)
   {
   case MENU_TYPE_EDIT_STR:
     return false;
     break;
   case MENU_TYPE_EDIT_NUM:
     return false;
     break;
   case MENU_TYPE_INFO:
     return false;
     break;
   case MENU_TYPE_SWITCH:
      return false;
      break;
   case MENU_TYPE_SELECT:
      return true;
      break;
   default:
      return false;
   }
   return false;
}

void Menu_Item::render_surface (string label, Properties* p, bool small)
{
   // only single entry menu items are editable
   if (small)
   {
      on.at(0) = TTF_RenderText_Blended(p->small_font, label.c_str(), p->on_color);
      off.at(0) = TTF_RenderText_Blended(p->small_font, label.c_str(), p->off_color);
   }
   else
   {
      on.at(0) = TTF_RenderText_Blended(p->font, label.c_str(), p->on_color);
      off.at(0) = TTF_RenderText_Blended(p->font, label.c_str(), p->off_color);
   }
}

void Menu::set_active(int pos)
{
   active = pos;
}


menuselect Menu::select_option(struct menuselect rt, int selected)
{
   rt.selected = items[selected].text[items[selected].subpos];
   for (unsigned int i=0; i<items.size(); i++)
   {
      if (items[i].is_editable_str())
         rt.strs.insert(make_pair(items[i].label, items[i].text[items[i].subpos]));
      
      else if (items[i].is_switch())
         rt.strs.insert(make_pair(items[i].label, items[i].text[items[i].subpos]));
      
      else if (items[i].is_editable_int())
         rt.ints.insert(make_pair(items[i].label, atoi(items[i].text[items[i].subpos].c_str())));
   }
   return rt;
}

menuselect Menu::select_option_static(struct menuselect rt, int selected)
{
   rt.selected = static_items[selected].text[static_items[selected].subpos];
   for (unsigned int i=0; i<static_items.size(); i++)
   {
      if (static_items[i].is_editable_str())
         rt.strs.insert(make_pair(static_items[i].label, static_items[i].text[static_items[i].subpos]));
      
      else if (static_items[i].is_switch())
         rt.strs.insert(make_pair(static_items[i].label, static_items[i].text[static_items[i].subpos]));
      
      else if (static_items[i].is_editable_int())
         rt.ints.insert(make_pair(static_items[i].label, atoi(static_items[i].text[static_items[i].subpos].c_str())));
   }
   return rt;
}

void Menu::edit_option(bool cursor_on)
{
   if (items[active].is_editable() && currently_editing == false)
   {
      currently_editing = true;
      redraw(currently_editing, cursor_on, false, false);
   }
   redraw(currently_editing, cursor_on, false, false);
}

menuselect Menu::display()
{
   SDL_Event event;
   SDL_EnableUNICODE(1);
   menuselect ret;
   bool cursor_on = true;

   redraw(currently_editing, cursor_on, false, false);

   while ( ret.selected == "void" )
   {
      SDL_Delay(4);
      if ( SDL_PollEvent(&event) )
      {
         if (!currently_editing)
         {
            redraw(currently_editing, cursor_on, false, false);
            switch (event.type)
            {
            case SDL_QUIT:
	      //ret.selected = "abort";
	      //return ret;
	      exit(0);
	      break;

            case SDL_MOUSEBUTTONDOWN:
               if (event.button.y >= y && event.button.y <= y+height*items.size())
               {
                  active = (event.button.y-y)/height;
                  if (items[active].is_selectable())
                     return select_option(ret, (event.button.y-y)/height);
                  if (items[active].is_editable())
                     edit_option(cursor_on);
                  if (items[active].is_switch() && (items[active].width() > 1))
                  {
                     if (event.button.x >= x-20 && event.button.x <= x)
                     {
                        items[active].subpos--;
                        if (items[active].subpos < 0) 
                           items[active].subpos = items[active].width()-1;
                     }
                     
                     if (event.button.x >= x+items[active].graphical_width(items[active].subpos) &&
                         event.button.x <= x+items[active].graphical_width(items[active].subpos)+20)
                     {
                        items[active].subpos++;
                        if (items[active].subpos > items[active].width()-1)
                           items[active].subpos = 0; 
                     }
                     
                     redraw(currently_editing, cursor_on, false, false);
                  }
               }
               break;

            case SDL_KEYDOWN:
            
               switch (event.key.keysym.sym)
               {
               case SDLK_1:
                  if (items.size() >= 1)
                     active = 0;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_2:
                  if (items.size() >= 2)
                     active = 1;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_3:
                  if (items.size() >= 3)
                     active = 2;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_4:
                  if (items.size() >= 4)
                     active = 3;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_5:
                  if (items.size() >= 5)
                     active = 4;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_6:
                  if (items.size() >= 6)
                     active = 5;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_7:
                  if (items.size() >= 7)
                     active = 6;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_8:
                  if (items.size() >= 8)
                     active = 7;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_9:
                  if (items.size() >= 9)
                     active = 8;
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_UP:
                  active--;
                  if (active < 0)
                     active = items.size()-1;
                  redraw(currently_editing, cursor_on, false, false);
                  break;
   
               case SDLK_DOWN:
                  active++;
                  if (active > (signed int)items.size()-1)
                     active = 0;

                  redraw(currently_editing, cursor_on, false, false);
                  break;
            
               case SDLK_LEFT:
                  if (items[active].width() > 1)
                  {
                     items[active].subpos--;
                     if (items[active].subpos == -1)
                        items[active].subpos = items[active].width()-1;
                  }
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_RIGHT:
                  if (items[active].width() > 1)
                  {
                     items[active].subpos++;
                     if (items[active].subpos > items[active].width()-1)
                        items[active].subpos = 0; 
                 }
                  redraw(currently_editing, cursor_on, false, false);
                  break;

               case SDLK_f:
                  SDL_WM_ToggleFullScreen(properties.screen);
                  break;
            
               case SDLK_RETURN:
                  if (items[active].is_selectable())
                     return select_option(ret, active);
                  edit_option(cursor_on);
                  break;
               default:
                  break;
               }
            }
         }
         
         else
         {
            redraw(currently_editing, cursor_on, false, false);

            if (cursor_on)
               cursor_on = false;
            else
               cursor_on = true;
            
            switch (event.type)
            {
            case SDL_QUIT:
               ret.selected = "abort";
               return ret;
               break;
            
            case SDL_KEYDOWN:
               if (event.key.keysym.sym ==SDLK_RETURN)
               {
                  if (immediate_return)
                  {
                     ret.selected = "Done";

                     for (unsigned int i=0; i<items.size(); i++)
                     {
                        if (items[i].is_editable_str())
                           ret.strs.insert(make_pair(items[i].label, items[i].text[items[i].subpos]));
                           
                        else if (items[i].is_switch())
                           ret.strs.insert(make_pair(items[i].label, items[i].text[items[i].subpos]));
                           
                        else if (items[i].is_editable_int())
                           ret.ints.insert(make_pair(items[i].label, atoi(items[i].text[items[i].subpos].c_str())));
                     }
                     return ret;
                  }
                  else
                     currently_editing = false;
               }
                  
               else if (event.key.keysym.sym ==SDLK_BACKSPACE)
               {
                  if (items[active].text[0].length() > 0)
                  {
                     items[active].text[0].erase(items[active].text[0].size()-1, 1);
                     items[active].render_surface(items[active].text[0], &properties, false);
                     redraw(currently_editing, cursor_on, false, false);
                  }
               }
                  
                     
               else if (event.key.keysym.sym == SDLK_LSHIFT)
                  ;
               
               else if (event.key.keysym.sym == SDLK_RSHIFT)
                  ;
               
               else
               {
                  char d = (char)(event.key.keysym.unicode);
                  items[active].text[0] = items[active].text[0] + d;
                  items[active].render_surface(items[active].text[0], &properties, false);
                  redraw(currently_editing, cursor_on, false, false);
               }
               items[active].render_surface(items[active].text.at(items[active].subpos), &properties, false);
               redraw(currently_editing, cursor_on, false, false);

               break;
            }
         }
      }
   
      // We never reach this :-)
   }
   return ret;
}


menuselect Menu::display_static(bool small)
{
   SDL_Event event;
   SDL_EnableUNICODE(1);
   menuselect ret;
   bool cursor_on = true;

   redraw(currently_editing, cursor_on, true, small);

   while ( ret.selected == "void" )
   {
      SDL_Delay(2);
      if ( SDL_PollEvent(&event) )
      {
         if (!currently_editing)
         {
            redraw(currently_editing, cursor_on, true, small);
            switch (event.type)
            {
            case SDL_QUIT:
	      //ret.selected = "abort";
	      //return ret;
	      exit(0);
	      break;

            case SDL_MOUSEBUTTONDOWN:
            {
               int x,y,w,h;
               for (unsigned int i=0; i<static_items.size(); i++)
               {
                  x = static_items.at(i).item_pos.x;
                  y = static_items.at(i).item_pos.y;
                  w = static_items.at(i).item_pos.w;
                  h = static_items.at(i).item_pos.h;

                  if (event.button.y >= y && event.button.y <= y+h && event.button.x >= x && event.button.x <= x+w)
                  {
                     if (static_items[active].is_selectable())
                        return select_option_static(ret, i);
                     edit_option(cursor_on);
                  }
               }
               
               break;
            }
            case SDL_KEYDOWN:
            
               switch (event.key.keysym.sym)
               {
               case SDLK_UP:
                  active--;
                  if (active < 0)
                     active = static_items.size()-1;
                  redraw(currently_editing, cursor_on, true, small);
                  break;
   
               case SDLK_DOWN:
                  active++;
                  if (active > (signed int)static_items.size()-1)
                     active = 0;

                  redraw(currently_editing, cursor_on, true, small);
                  break;
            
               case SDLK_f:
                  SDL_WM_ToggleFullScreen(properties.screen);
                  break;
            
               case SDLK_RETURN:
                  if (static_items[active].is_selectable())
                     return select_option_static(ret, active);
                  edit_option(cursor_on);
                  break;
                  
               default:
                  break;
               }
            }
         }
      }
   
      // We never reach this :-)
   }
   return ret;
}










