/*
 *  replacecol.cc
 *
 *  These three functions are part of the SDL documentation, the
 *  exact authors couldn't be determined.
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <SDL.h>

Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
  int bpp = surface->format->BytesPerPixel;
  /* Here p is the address to the pixel we want to retrieve */
  Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
  
  switch(bpp) {
  case 1:
    return *p;
    
  case 2:
    return *(Uint16 *)p;
    
  case 3:
    if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
      return p[0] << 16 | p[1] << 8 | p[2];
    else
      return p[0] | p[1] << 8 | p[2] << 16;
    
  case 4:
    return *(Uint32 *)p;
    
  default:
    return 0;       /* shouldn't happen, but avoids warnings */
  }
}

/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
  int bpp = surface->format->BytesPerPixel;
  /* Here p is the address to the pixel we want to set */
  Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
  
  switch(bpp) {
  case 1:
    *p = pixel;
    break;
    
  case 2:
    *(Uint16 *)p = pixel;
    break;
    
  case 3:
    if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
      p[0] = (pixel >> 16) & 0xff;
      p[1] = (pixel >> 8) & 0xff;
      p[2] = pixel & 0xff;
    } else {
      p[0] = pixel & 0xff;
      p[1] = (pixel >> 8) & 0xff;
      p[2] = (pixel >> 16) & 0xff;
    }
    break;
    
  case 4:
    *(Uint32 *)p = pixel;
    break;
  }
}


void replacered(SDL_Surface *surface,int TR, int TG, int TB)
{
  
  /* Extracting color components from a 32-bit color value */
  SDL_PixelFormat *fmt;
  Uint32 temp, pixel;
  Uint8 red, green, blue, alpha;
  
  fmt=surface->format;
  
  for (int x=0;x<surface->w;x++){
    for (int y=0;y<surface->h;y++){
      
      SDL_LockSurface(surface);
      pixel=getpixel(surface,x,y);
      SDL_UnlockSurface(surface);
      
      /* Get Red component */
      temp=pixel&fmt->Rmask; /* Isolate red component */
      temp=temp>>fmt->Rshift;/* Shift it down to 8-bit */
      temp=temp<<fmt->Rloss; /* Expand to a full 8-bit number */
      red=(Uint8)temp;
      
      /* Get Green component */
      temp=pixel&fmt->Gmask; /* Isolate green component */
      temp=temp>>fmt->Gshift;/* Shift it down to 8-bit */
      temp=temp<<fmt->Gloss; /* Expand to a full 8-bit number */
      green=(Uint8)temp;
	
      /* Get Blue component */
      temp=pixel&fmt->Bmask; /* Isolate blue component */
      temp=temp>>fmt->Bshift;/* Shift it down to 8-bit */
      temp=temp<<fmt->Bloss; /* Expand to a full 8-bit number */
      blue=(Uint8)temp;
	
      /* Get Alpha component */
      temp=pixel&fmt->Amask; /* Isolate alpha component */
      temp=temp>>fmt->Ashift;/* Shift it down to 8-bit */
      temp=temp<<fmt->Aloss; /* Expand to a full 8-bit number */
      alpha=(Uint8)temp;

      // change < > and values to make this generic
      // this is a hack to match the red shades in the current sprite set...
      if ((red > 100 && green == 0 && blue == 0) 
	  || (red > 220 && green > 110 && green < 130)
	  || (red > 150 && red < 200 && green < 25 && blue == 0)) {
	Uint32 newcolor = SDL_MapRGB(surface->format,TR, TG, TB);
	SDL_LockSurface(surface);
	putpixel(surface,x,y,newcolor);
	SDL_UnlockSurface(surface);
      }
    }
  }
}








