/*
 *  seabattle.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "seabattle.h"
#include <time.h>

using namespace std;

Seabattle::Seabattle(SDL_Surface* s,Player* a, Player* b):
  windrichtung(0),
  windstaerke(rand() % 8),
  windstetigkeit(10),
  wechselstimmung(0),
  screen(s),
  end_game(false),
  entered(false),
  p1(a,true),
  p2(b,false),
  mode(AI_CONFUSED)
{}

Seabattle::~Seabattle(){
}

void Seabattle::handle_events()
{
   SDL_Event event;
   Uint8* keys;
   while ( SDL_PollEvent(&event) )
   {
      switch (event.type)
      {
      case SDL_QUIT:
         end_game=true;
         break;
      default:
         break;
      }
   }
   keys = SDL_GetKeyState(0);
   if (keys[SDLK_UP])
   {
     p1.ship->canvas_modify(1);
   }
   if (keys[SDLK_DOWN])
   {
     p1.ship->canvas_modify(-1);
   }
   if (keys[SDLK_LEFT])
   {
     p1.ship->rotate(-10);
   }
   if (keys[SDLK_RIGHT])
   {
     p1.ship->rotate(10);
   }
   if (keys[SDLK_LCTRL])
   {
     if (SDL_GetTicks()>(p1.lastshot_time+p1.player->get_ship()->get_delay()) && p1.firepower<47 && p1.deathcounter<1 ) p1.firepower++;
   }
   if (!keys[SDLK_LCTRL] && p1.firepower!=0)
   {
     cannonballs.addShot(&p1);
   }

   bool cheats_which_are_purely_for_the_purpose_of_easier_testing = false;

   if (cheats_which_are_purely_for_the_purpose_of_easier_testing)
   {
      if (keys[SDLK_z])
      {
         p1.player->get_ship()->add_damage();
      }

      if (keys[SDLK_u])
      {
         p2.player->get_ship()->add_damage();
      }

      if (keys[SDLK_i])
      {
         entered=true;
         end_game=true;
      }
   }

   // Player2 Controls for Multiplayer Mode

   if (!p2.player->is_ai()){
     
     if ( keys[SDLK_w] )
       {
	 p2.ship->canvas_modify(1);
       }
     if (keys[SDLK_s])
       {
	 p2.ship->canvas_modify(-1);
       }
     if (keys[SDLK_a])
       {
	 p2.ship->rotate(-10);
       }
     if (keys[SDLK_d])
       {
	 p2.ship->rotate(10);
       }
     if (keys[SDLK_RCTRL])
       {
	 if (SDL_GetTicks()>(p2.lastshot_time+p2.player->get_ship()->get_delay()) && p2.firepower<47 && p2.deathcounter<1) p2.firepower++;
       }
     if (!keys[SDLK_RCTRL] && p2.firepower!=0)
       {
	 cannonballs.addShot(&p2);
       }
   }
   

//   if ( keys[SDLK_ESCAPE] )
//      end_game=true;

   if ( keys[SDLK_RETURN] || keys[SDLK_TAB] ) enter();
   if ( keys[SDLK_f] )
   {
      SDL_WM_ToggleFullScreen(screen);
   }   
  }

void Seabattle::enter()
{
  Tools t;
  if ((abs(p1.ship->get_x()-p2.ship->get_x())<20) &&
      (abs(p1.ship->get_y()-p2.ship->get_y())<20) &&
      p1.deathcounter<1 && p2.deathcounter<1)
  {
     tarzan=t.load_wav("tarzan.wav");
     t.play_sound(tarzan);
     Mix_FreeChunk(tarzan);
     SDL_Delay(4000);
     entered=true;
     end_game=true;
  }
}

void Seabattle::redraw_ships()
{
  p1.ship->travel(screen->w, screen->h, windstaerke, windrichtung, p2.ship);
  p2.ship->travel(screen->w, screen->h, windstaerke, windrichtung, p1.ship);

  if(p1.deathcounter<100) p1.ship->draw(screen);
  if(p2.deathcounter<100) p2.ship->draw(screen);
}

void Seabattle::wait_ticks()
{
   while ( last_tick > SDL_GetTicks()-40 ){
     SDL_Delay(20);
   }
   last_tick = SDL_GetTicks();
}

void Seabattle::draw_windspeed()
{
  wechselstimmung = rand() % 100;
  
  if (wechselstimmung < windstetigkeit)
    {
      if ((wechselstimmung % 2) == 1)
	  windstaerke++;
      else
	windstaerke--;
    }

  if (windstaerke < 0)
     windstaerke = 0;
  if (windstaerke > 12)
     windstaerke = 12;
      
  wechselstimmung = rand() % 100;
  
  if (wechselstimmung < windstetigkeit)
    {
      if ((wechselstimmung % 2) == 1)
	windrichtung++;
      else
	windrichtung--;
    }

  windrichtung += 36;
  windrichtung = windrichtung % 36;

  // draw windspeed
  Uint32 col = SDL_MapRGB(screen->format, 255,0,0);
  SDL_Color text_color;
  SDL_GetRGB(col, screen->format, &text_color.r, &text_color.g, &text_color.b);
  
  col = SDL_MapRGB(screen->format, 20,66,95);
  SDL_Color bg_color;
  SDL_GetRGB(col, screen->format, &bg_color.r, &bg_color.g, &bg_color.b);
 
  stringstream windspeed;
  windspeed << "Wind Speed " << windstaerke;

  SDL_Surface* text; 

  text = TTF_RenderText_Shaded(score_font, windspeed.str().c_str(), text_color, bg_color );

  SDL_SetColorKey (text,SDL_SRCCOLORKEY,SDL_MapRGB(text->format,20,66,95));
  SDL_BlitSurface(text, 0 , screen, NULL );
  SDL_FreeSurface(text);

  // draw windrose
  SDL_Rect rose_pos;
  rose_pos.h=139;
  rose_pos.w=139;
  rose_pos.x=0;
  rose_pos.y=341;

  SDL_SetColorKey (windrose[windrichtung],SDL_SRCCOLORKEY,SDL_MapRGB(windrose[windrichtung]->format,0,0,255));
  SDL_BlitSurface(windrose[windrichtung], 0, screen, &rose_pos);
}


void Seabattle::draw_playernames()
{
  SDL_Color local_color;
  SDL_GetRGB(SDL_MapRGB(screen->format, p1.ship->r,p1.ship->g,p1.ship->b), screen->format, &local_color.r, &local_color.g, &local_color.b);
  
  SDL_Color remote_color;
  SDL_GetRGB(SDL_MapRGB(screen->format, p2.ship->r,p2.ship->g,p2.ship->b), screen->format, &remote_color.r, &remote_color.g, &remote_color.b);
  
  SDL_Color bg_color;
  SDL_GetRGB(SDL_MapRGB(screen->format, 20,70,100), screen->format, &bg_color.r, &bg_color.g, &bg_color.b);
 
  stringstream local_text;
  stringstream remote_text;

  local_text << p1.player->get_name() << " - Condition: " << p1.player->get_ship()->get_state() << "%";
  remote_text << p2.player->get_name() << " - Condition: " << p2.player->get_ship()->get_state() << "%";

  SDL_Surface* localt;
  SDL_Surface* remotet;
  SDL_Rect local_rect;
  SDL_Rect remote_rect;
  local_rect.x=0;
  local_rect.y=30;
  remote_rect.x=0;
  remote_rect.y=55;
  
  localt = TTF_RenderText_Shaded(score_font, local_text.str().c_str(), local_color, bg_color );
  SDL_BlitSurface(localt, 0 , screen, &local_rect );
  SDL_FreeSurface(localt);

  remotet = TTF_RenderText_Shaded(score_font, remote_text.str().c_str(), remote_color, bg_color );
  SDL_BlitSurface(remotet, 0 , screen, &remote_rect );
  SDL_FreeSurface(remotet);
}

int Seabattle::game_play()
{
  lastshot_time=SDL_GetTicks();
  round_time=SDL_GetTicks(); 

  line_pos.x=640-184;
  line_pos.y=0;
  line_pos.h=179;
  line_pos.w=184;

  waterframestep=1;
  waterframe=0;

  while ( ! end_game )
    {
      waterframe+=waterframestep;
      if (waterframe>=(int)water_screens.size()-1) waterframestep=-1;
      if (waterframe<1) waterframestep=1;
      handle_events();
      if (p2.player->is_ai()) ai();
      SDL_BlitSurface(water_screens[waterframe], 0, screen, 0);
      draw_windspeed();
      draw_playernames();
      redraw_ships();

      p1.draw_firepower(screen);
      p2.draw_firepower(screen);
      
      p1.explode(screen);
      p2.explode(screen);

      if (cannonballs.isDead(&p1)) {
	p1.deathcounter++;
      }
      if (cannonballs.isDead(&p2)) {
	p2.deathcounter++;
      }
      // ship is sinking 99 frames after fatal hit
      if (p1.deathcounter==99) cannonballs.splashes.addSplash(p1.ship->get_x()+35,p1.ship->get_y()+35);
      if (p2.deathcounter==99) cannonballs.splashes.addSplash(p2.ship->get_x()+35,p2.ship->get_y()+35);
      // game over 41 frames later
      if (p1.deathcounter==140) return 2;
      if (p2.deathcounter==140) return 1;

      cannonballs.drawShots(screen);
      SDL_Flip(screen);

      wait_ticks();
    }
  if (entered) {
    return 3; //entered 
  } else {
    return 0; // surrendered
  }
}

void init_nutshell(Seabattle* battle)
{
   Tools t;

   vector<string> ship1;
   vector<string> ship2;
   vector<string> ship3;
   vector<string> ship4;
   vector<string> ship5;
   vector<string> ship6;
   vector<string> ship7;
 
   ship1.push_back("rboat_1_1.png");
   ship1.push_back("rboat_1_2.png");
   ship1.push_back("rboat_1_3.png");
   ship1.push_back("rboat_1_4.png");
   ship1.push_back("rboat_1_5.png");
   ship1.push_back("rboat_1_6.png");
   ship1.push_back("rboat_1_7.png");
   ship1.push_back("rboat_1_8.png");
   ship1.push_back("rboat_1_9.png");
   ship1.push_back("rboat_1_10.png");
   ship1.push_back("rboat_1_11.png");
   ship1.push_back("rboat_1_12.png");
   ship1.push_back("rboat_1_13.png");
   ship1.push_back("rboat_1_14.png");
   ship1.push_back("rboat_1_15.png");
   ship1.push_back("rboat_1_16.png");
   ship1.push_back("rboat_1_17.png");
   ship1.push_back("rboat_1_18.png");
   ship1.push_back("rboat_1_19.png");
   ship1.push_back("rboat_1_20.png");
   ship1.push_back("rboat_1_21.png");
   ship1.push_back("rboat_1_22.png");
   ship1.push_back("rboat_1_23.png");
   ship1.push_back("rboat_1_24.png");
   ship1.push_back("rboat_1_25.png");
   ship1.push_back("rboat_1_26.png");
   ship1.push_back("rboat_1_27.png");
   ship1.push_back("rboat_1_28.png");
   ship1.push_back("rboat_1_29.png");
   ship1.push_back("rboat_1_30.png");
   ship1.push_back("rboat_1_31.png");
   ship1.push_back("rboat_1_32.png");
   ship1.push_back("rboat_1_33.png");
   ship1.push_back("rboat_1_34.png");
   ship1.push_back("rboat_1_35.png");
   ship1.push_back("rboat_1_36.png");
 
   ship2.push_back("rboat_2_1.png");
   ship2.push_back("rboat_2_2.png");
   ship2.push_back("rboat_2_3.png");
   ship2.push_back("rboat_2_4.png");
   ship2.push_back("rboat_2_5.png");
   ship2.push_back("rboat_2_6.png");
   ship2.push_back("rboat_2_7.png");
   ship2.push_back("rboat_2_8.png");
   ship2.push_back("rboat_2_9.png");
   ship2.push_back("rboat_2_10.png");
   ship2.push_back("rboat_2_11.png");
   ship2.push_back("rboat_2_12.png");
   ship2.push_back("rboat_2_13.png");
   ship2.push_back("rboat_2_14.png");
   ship2.push_back("rboat_2_15.png");
   ship2.push_back("rboat_2_16.png");
   ship2.push_back("rboat_2_17.png");
   ship2.push_back("rboat_2_18.png");
   ship2.push_back("rboat_2_19.png");
   ship2.push_back("rboat_2_20.png");
   ship2.push_back("rboat_2_21.png");
   ship2.push_back("rboat_2_22.png");
   ship2.push_back("rboat_2_23.png");
   ship2.push_back("rboat_2_24.png");
   ship2.push_back("rboat_2_25.png");
   ship2.push_back("rboat_2_26.png");
   ship2.push_back("rboat_2_27.png");
   ship2.push_back("rboat_2_28.png");
   ship2.push_back("rboat_2_29.png");
   ship2.push_back("rboat_2_30.png");
   ship2.push_back("rboat_2_31.png");
   ship2.push_back("rboat_2_32.png");
   ship2.push_back("rboat_2_33.png");
   ship2.push_back("rboat_2_34.png");
   ship2.push_back("rboat_2_35.png");
   ship2.push_back("rboat_2_36.png");
 
   ship3.push_back("rboat_3_1.png");
   ship3.push_back("rboat_3_2.png");
   ship3.push_back("rboat_3_3.png");
   ship3.push_back("rboat_3_4.png");
   ship3.push_back("rboat_3_5.png");
   ship3.push_back("rboat_3_6.png");
   ship3.push_back("rboat_3_7.png");
   ship3.push_back("rboat_3_8.png");
   ship3.push_back("rboat_3_9.png");
   ship3.push_back("rboat_3_10.png");
   ship3.push_back("rboat_3_11.png");
   ship3.push_back("rboat_3_12.png");
   ship3.push_back("rboat_3_13.png");
   ship3.push_back("rboat_3_14.png");
   ship3.push_back("rboat_3_15.png");
   ship3.push_back("rboat_3_16.png");
   ship3.push_back("rboat_3_17.png");
   ship3.push_back("rboat_3_18.png");
   ship3.push_back("rboat_3_19.png");
   ship3.push_back("rboat_3_20.png");
   ship3.push_back("rboat_3_21.png");
   ship3.push_back("rboat_3_22.png");
   ship3.push_back("rboat_3_23.png");
   ship3.push_back("rboat_3_24.png");
   ship3.push_back("rboat_3_25.png");
   ship3.push_back("rboat_3_26.png");
   ship3.push_back("rboat_3_27.png");
   ship3.push_back("rboat_3_28.png");
   ship3.push_back("rboat_3_29.png");
   ship3.push_back("rboat_3_30.png");
   ship3.push_back("rboat_3_31.png");
   ship3.push_back("rboat_3_32.png");
   ship3.push_back("rboat_3_33.png");
   ship3.push_back("rboat_3_34.png");
   ship3.push_back("rboat_3_35.png");
   ship3.push_back("rboat_3_36.png");
 
   ship4.push_back("rboat_4_1.png");
   ship4.push_back("rboat_4_2.png");
   ship4.push_back("rboat_4_3.png");
   ship4.push_back("rboat_4_4.png");
   ship4.push_back("rboat_4_5.png");
   ship4.push_back("rboat_4_6.png");
   ship4.push_back("rboat_4_7.png");
   ship4.push_back("rboat_4_8.png");
   ship4.push_back("rboat_4_9.png");
   ship4.push_back("rboat_4_10.png");
   ship4.push_back("rboat_4_11.png");
   ship4.push_back("rboat_4_12.png");
   ship4.push_back("rboat_4_13.png");
   ship4.push_back("rboat_4_14.png");
   ship4.push_back("rboat_4_15.png");
   ship4.push_back("rboat_4_16.png");
   ship4.push_back("rboat_4_17.png");
   ship4.push_back("rboat_4_18.png");
   ship4.push_back("rboat_4_19.png");
   ship4.push_back("rboat_4_20.png");
   ship4.push_back("rboat_4_21.png");
   ship4.push_back("rboat_4_22.png");
   ship4.push_back("rboat_4_23.png");
   ship4.push_back("rboat_4_24.png");
   ship4.push_back("rboat_4_25.png");
   ship4.push_back("rboat_4_26.png");
   ship4.push_back("rboat_4_27.png");
   ship4.push_back("rboat_4_28.png");
   ship4.push_back("rboat_4_29.png");
   ship4.push_back("rboat_4_30.png");
   ship4.push_back("rboat_4_31.png");
   ship4.push_back("rboat_4_32.png");
   ship4.push_back("rboat_4_33.png");
   ship4.push_back("rboat_4_34.png");
   ship4.push_back("rboat_4_35.png");
   ship4.push_back("rboat_4_36.png");
 
   ship5.push_back("rboat_5_1.png");
   ship5.push_back("rboat_5_2.png");
   ship5.push_back("rboat_5_3.png");
   ship5.push_back("rboat_5_4.png");
   ship5.push_back("rboat_5_5.png");
   ship5.push_back("rboat_5_6.png");
   ship5.push_back("rboat_5_7.png");
   ship5.push_back("rboat_5_8.png");
   ship5.push_back("rboat_5_9.png");
   ship5.push_back("rboat_5_10.png");
   ship5.push_back("rboat_5_11.png");
   ship5.push_back("rboat_5_12.png");
   ship5.push_back("rboat_5_13.png");
   ship5.push_back("rboat_5_14.png");
   ship5.push_back("rboat_5_15.png");
   ship5.push_back("rboat_5_16.png");
   ship5.push_back("rboat_5_17.png");
   ship5.push_back("rboat_5_18.png");
   ship5.push_back("rboat_5_19.png");
   ship5.push_back("rboat_5_20.png");
   ship5.push_back("rboat_5_21.png");
   ship5.push_back("rboat_5_22.png");
   ship5.push_back("rboat_5_23.png");
   ship5.push_back("rboat_5_24.png");
   ship5.push_back("rboat_5_25.png");
   ship5.push_back("rboat_5_26.png");
   ship5.push_back("rboat_5_27.png");
   ship5.push_back("rboat_5_28.png");
   ship5.push_back("rboat_5_29.png");
   ship5.push_back("rboat_5_30.png");
   ship5.push_back("rboat_5_31.png");
   ship5.push_back("rboat_5_32.png");
   ship5.push_back("rboat_5_33.png");
   ship5.push_back("rboat_5_34.png");
   ship5.push_back("rboat_5_35.png");
   ship5.push_back("rboat_5_36.png");
 
   ship6.push_back("rboat_6_1.png");
   ship6.push_back("rboat_6_2.png");
   ship6.push_back("rboat_6_3.png");
   ship6.push_back("rboat_6_4.png");
   ship6.push_back("rboat_6_5.png");
   ship6.push_back("rboat_6_6.png");
   ship6.push_back("rboat_6_7.png");
   ship6.push_back("rboat_6_8.png");
   ship6.push_back("rboat_6_9.png");
   ship6.push_back("rboat_6_10.png");
   ship6.push_back("rboat_6_11.png");
   ship6.push_back("rboat_6_12.png");
   ship6.push_back("rboat_6_13.png");
   ship6.push_back("rboat_6_14.png");
   ship6.push_back("rboat_6_15.png");
   ship6.push_back("rboat_6_16.png");
   ship6.push_back("rboat_6_17.png");
   ship6.push_back("rboat_6_18.png");
   ship6.push_back("rboat_6_19.png");
   ship6.push_back("rboat_6_20.png");
   ship6.push_back("rboat_6_21.png");
   ship6.push_back("rboat_6_22.png");
   ship6.push_back("rboat_6_23.png");
   ship6.push_back("rboat_6_24.png");
   ship6.push_back("rboat_6_25.png");
   ship6.push_back("rboat_6_26.png");
   ship6.push_back("rboat_6_27.png");
   ship6.push_back("rboat_6_28.png");
   ship6.push_back("rboat_6_29.png");
   ship6.push_back("rboat_6_30.png");
   ship6.push_back("rboat_6_31.png");
   ship6.push_back("rboat_6_32.png");
   ship6.push_back("rboat_6_33.png");
   ship6.push_back("rboat_6_34.png");
   ship6.push_back("rboat_6_35.png");
   ship6.push_back("rboat_6_36.png");
 
   ship7.push_back("rboat_7_1.png");
   ship7.push_back("rboat_7_2.png");
   ship7.push_back("rboat_7_3.png");
   ship7.push_back("rboat_7_4.png");
   ship7.push_back("rboat_7_5.png");
   ship7.push_back("rboat_7_6.png");
   ship7.push_back("rboat_7_7.png");
   ship7.push_back("rboat_7_8.png");
   ship7.push_back("rboat_7_9.png");
   ship7.push_back("rboat_7_10.png");
   ship7.push_back("rboat_7_11.png");
   ship7.push_back("rboat_7_12.png");
   ship7.push_back("rboat_7_13.png");
   ship7.push_back("rboat_7_14.png");
   ship7.push_back("rboat_7_15.png");
   ship7.push_back("rboat_7_16.png");
   ship7.push_back("rboat_7_17.png");
   ship7.push_back("rboat_7_18.png");
   ship7.push_back("rboat_7_19.png");
   ship7.push_back("rboat_7_20.png");
   ship7.push_back("rboat_7_21.png");
   ship7.push_back("rboat_7_22.png");
   ship7.push_back("rboat_7_23.png");
   ship7.push_back("rboat_7_24.png");
   ship7.push_back("rboat_7_25.png");
   ship7.push_back("rboat_7_26.png");
   ship7.push_back("rboat_7_27.png");
   ship7.push_back("rboat_7_28.png");
   ship7.push_back("rboat_7_29.png");
   ship7.push_back("rboat_7_30.png");
   ship7.push_back("rboat_7_31.png");
   ship7.push_back("rboat_7_32.png");
   ship7.push_back("rboat_7_33.png");
   ship7.push_back("rboat_7_34.png");
   ship7.push_back("rboat_7_35.png");
   ship7.push_back("rboat_7_36.png");
  
   battle->rboat.push_back(ship1);
   battle->rboat.push_back(ship2);
   battle->rboat.push_back(ship3);
   battle->rboat.push_back(ship4);
   battle->rboat.push_back(ship5);
   battle->rboat.push_back(ship6);
   battle->rboat.push_back(ship7);

   ship1.clear();
   ship2.clear();
   ship3.clear();
   ship4.clear();
   ship5.clear();
   ship6.clear();
   ship6.clear();

   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser1.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser2.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser3.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser4.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser5.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser6.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser7.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser8.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser9.png")));
   battle->water_screens.push_back(t.load_img(t.sprites_path("wasser10.png")));

   battle->windrose.push_back(t.load_img(t.sprites_path("rose1.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose2.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose3.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose4.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose5.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose6.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose7.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose8.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose9.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose10.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose11.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose12.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose13.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose14.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose15.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose16.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose17.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose18.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose19.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose20.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose21.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose22.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose23.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose24.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose25.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose26.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose27.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose28.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose29.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose30.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose31.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose32.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose33.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose34.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose35.png")));
   battle->windrose.push_back(t.load_img(t.sprites_path("rose36.png")));
   
   battle->p1.explosion = t.load_img(t.sprites_path("bombexplosion.png"));
   SDL_SetColorKey (battle->p1.explosion,SDL_SRCCOLORKEY,SDL_MapRGB(battle->p1.explosion->format,255,0,255));

   battle->p2.explosion = t.load_img(t.sprites_path("bombexplosion.png"));
   SDL_SetColorKey (battle->p2.explosion,SDL_SRCCOLORKEY,SDL_MapRGB(battle->p2.explosion->format,255,0,255));

   battle->p1.ship = new NutShip(battle->rboat,600,50,0,0,180,3,20,70,100);
   battle->p2.ship = new NutShip(battle->rboat,20,240,0,0,0,3,20,70,100);
   
   //setting colors
   battle->p1.setcolor();
   battle->p2.setcolor();

   SDL_Rect field;
   field.x = 0;
   field.y = 0;
   field.w = 640;
   field.h = 480;

   battle->score_font = t.load_ttf(18, "vera.ttf");

   battle->last_tick = SDL_GetTicks();
}

int Seabattle::run(){

  srand(time(NULL));
  init_nutshell(this);
  int retval=game_play();

  // cleanup
  TTF_CloseFont(score_font);
  for (vector<SDL_Surface*>:: iterator ws = water_screens.begin(); ws != water_screens.end(); ws++) {
    SDL_FreeSurface(*ws);
  }  
  water_screens.clear();
  for (vector<SDL_Surface*>:: iterator wr = windrose.begin(); wr != windrose.end(); wr++) {
    SDL_FreeSurface(*wr);
  }
  windrose.clear();
  p1.unload();
  p2.unload();

  rboat.clear();
  cannonballs.unload();

  return retval;
}


void Seabattle::ai(){

  cross_angle=360-(int)(atan2((double)(p1.ship->get_x()-p2.ship->get_x()),
			      (double)(p1.ship->get_y()-p2.ship->get_y()))*180/PI+270)%360;
  aim_angle=360-(int)(atan2((double)(p1.ship->get_x()-p2.ship->get_x()),
			    (double)(p1.ship->get_y()-p2.ship->get_y()))*180/PI+180)%360;
  escape_angle=360-(int)(atan2((double)(p1.ship->get_x()-p2.ship->get_x()),
			       (double)(p1.ship->get_y()-p2.ship->get_y()))*180/PI+90)%360;
  
  ccw_deg=(p2.ship->get_a()+360-cross_angle)%360;
  cw_deg=(360-p2.ship->get_a()+cross_angle)%360;
  
  a_ccw_deg=(p2.ship->get_a()+360-aim_angle)%360;
  a_cw_deg=(360-p2.ship->get_a()+aim_angle)%360;

  e_ccw_deg=(p2.ship->get_a()+360-escape_angle)%360;
  e_cw_deg=(360-p2.ship->get_a()+escape_angle)%360;
  
  distance=(int)sqrt((double)((abs(p1.ship->get_x()-p2.ship->get_x()))*
			      (abs(p1.ship->get_x()-p2.ship->get_x()))+
			      (abs(p1.ship->get_y()-p2.ship->get_y()))*
			      (abs(p1.ship->get_y()-p2.ship->get_y()))));
  
  int rnd=rand()%50;
  if(rnd==1) p2.ship->canvas_modify(1);
  if(rnd==2) p2.ship->canvas_modify(-1);  
  if(rnd==3) p2.lastshot_time=SDL_GetTicks();

  int rnd2=rand()%200;
  if (rnd2==0){
    int rnd3=rand()%10;
    if (p2.player->get_ship()->boarding_is_worthwhile(p1.player->get_ship())) {
      if (rnd3>2) mode=AI_ENTER;
      if (rnd3==1) mode=AI_ATTACK;
    } else {
      if (rnd3>2) mode=AI_ATTACK;
      if (rnd3==1) mode=AI_ENTER;    
    }
    if (rnd3==0) mode=AI_CONFUSED;
    if (rnd3==2) mode=AI_ESCAPE;

  }

  switch(mode){
  case AI_ATTACK:
    if (a_ccw_deg>5 && a_cw_deg>5){
      if (a_ccw_deg > a_cw_deg) {
	p2.ship->rotate(2);
      } else {
	p2.ship->rotate(-2);      
      }
    }
    if (SDL_GetTicks()>(p2.lastshot_time+p2.player->get_ship()->get_delay()) && p2.deathcounter<1 ) {
      p2.firepower=distance/10+1;
      cannonballs.addShot(&p2);
    }
    break;
  case AI_ENTER:
    if (ccw_deg>5 && cw_deg>5){
      if (ccw_deg > cw_deg) {
	p2.ship->rotate(2);
      } else {
	p2.ship->rotate(-2);      
      }
    }
    enter();
    break;
  case AI_ESCAPE:
    if (e_ccw_deg>5 && e_cw_deg>5){
      if (e_ccw_deg > e_cw_deg) {
	p2.ship->rotate(2);
      } else {
	p2.ship->rotate(-2);      
      }
    }
    break;
  case AI_CONFUSED:
    if(rnd<3) p2.ship->rotate(-10);
    if(rnd==4) p2.ship->rotate(10);
    if (SDL_GetTicks()>(p2.lastshot_time+p2.player->get_ship()->get_delay()) && p2.deathcounter<1 ) {
      p2.firepower=distance/10+1;
      cannonballs.addShot(&p2);
    }
    break;
  }
  
}














