/*
 *  ship.cc
 *
 *  Copyright (C) 2004 Moritz Mhlenhoff
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string>
#include "good.h"
#include "ship.h"
#include "math.h"
#include "tools.h"

using namespace std;

Ship::Ship(string shiptype, string shipname, int shipstate)
{
   Good sugar;
   Good rum;
   Good tobacco;
   Good ammo;
   Good weapons;

   Tools t;
   gameplay_parameters = readdata(t.data_path("gameplay-parameters.conf"));

   sugar.set_name("Sugar");
   rum.set_name("Rum");
   tobacco.set_name("Tobacco");
   ammo.set_name("Ammo");
   weapons.set_name("Weapons");

   goods.push_back(sugar);
   goods.push_back(rum);
   goods.push_back(tobacco);
   goods.push_back(ammo);
   goods.push_back(weapons);

   name = shipname;
   state = shipstate;
   crew_members = 0;
   cannons = 0;
   cargo = 0;
   crew_pay = 0;
   crew_content = 0;
   delay = 3000;
   pay_percent = 100;

   crew_content = 100;
   
   // bogus
   ammunition = 0;
   type = shiptype;
   starving = false;
   food = 0.0;

   ammo_capacity = 100;
   cannon_range = 100;

   if (shiptype == "SCHOONER")
   {
      plating = gameplay_parameters.get_int("schooner_initial_plating");
      speed_capacity = gameplay_parameters.get_int("schooner_speed_capacity");
      cargo_capacity = gameplay_parameters.get_int("schooner_speed_capacity");
      crew_optimum = gameplay_parameters.get_int("schooner_crew_optimum");
      cannon_capacity = gameplay_parameters.get_int("schooner_cannon_capacity");
      plating_capacity = gameplay_parameters.get_int("schooner_plating_capacity");
      delay_capacity = gameplay_parameters.get_int("schooner_delay_capacity");
      food_capacity = gameplay_parameters.get_int("schooner_food_capacity");
   }

   if (shiptype == "BRIG")
   {
      plating = gameplay_parameters.get_int("brig_initial_plating");
      speed_capacity = gameplay_parameters.get_int("brig_speed_capacity");
      cargo_capacity = gameplay_parameters.get_int("brig_speed_capacity");
      crew_optimum = gameplay_parameters.get_int("brig_crew_optimum");
      cannon_capacity = gameplay_parameters.get_int("brig_cannon_capacity");
      plating_capacity = gameplay_parameters.get_int("brig_plating_capacity");
      delay_capacity = gameplay_parameters.get_int("brig_delay_capacity");
      food_capacity = gameplay_parameters.get_int("brig_food_capacity");
   }

   if (shiptype == "FRIGATE")
   {
      plating = gameplay_parameters.get_int("frigate_initial_plating");
      speed_capacity = gameplay_parameters.get_int("frigate_speed_capacity");
      cargo_capacity = gameplay_parameters.get_int("frigate_speed_capacity");
      crew_optimum = gameplay_parameters.get_int("frigate_crew_optimum");
      cannon_capacity = gameplay_parameters.get_int("frigate_cannon_capacity");
      plating_capacity = gameplay_parameters.get_int("frigate_plating_capacity");
      delay_capacity = gameplay_parameters.get_int("frigate_delay_capacity");
      food_capacity = gameplay_parameters.get_int("frigate_food_capacity");
   }

   if (shiptype == "MERCHANT")
   {
      plating = gameplay_parameters.get_int("merchant_initial_plating");
      speed_capacity = gameplay_parameters.get_int("merchant_speed_capacity");
      cargo_capacity = gameplay_parameters.get_int("merchant_speed_capacity");
      crew_optimum = gameplay_parameters.get_int("merchant_crew_optimum");
      cannon_capacity = gameplay_parameters.get_int("merchant_cannon_capacity");
      plating_capacity = gameplay_parameters.get_int("merchant_plating_capacity");
      delay_capacity = gameplay_parameters.get_int("merchant_delay_capacity");
      food_capacity = gameplay_parameters.get_int("merchant_food_capacity");
   }
   
   if (shiptype == "GALLEON")
   {
      plating = gameplay_parameters.get_int("galleon_initial_plating");
      speed_capacity = gameplay_parameters.get_int("galleon_speed_capacity");
      cargo_capacity = gameplay_parameters.get_int("galleon_speed_capacity");
      crew_optimum = gameplay_parameters.get_int("galleon_crew_optimum");
      cannon_capacity = gameplay_parameters.get_int("galleon_cannon_capacity");
      plating_capacity = gameplay_parameters.get_int("galleon_plating_capacity");
      delay_capacity = gameplay_parameters.get_int("galleon_delay_capacity");
      food_capacity = gameplay_parameters.get_int("galleon_food_capacity");
   }

   if (shiptype == "NONE")
   {
      plating = 0;
      speed_capacity = 0;
      cargo_capacity = 0;
      crew_optimum = 0;
      cannon_capacity = 0;
      plating_capacity = 0;
      delay_capacity = 2000;
      food_capacity = 0;
   }
}

Ship::Ship(Savegame* sg)
{
   Tools t;
   gameplay_parameters = readdata(t.data_path("gameplay-parameters.conf"));

   food = sg->read_float();
   for (int i=0; i<5; i++)
      goods.push_back(Good(sg));
   flag = sg->read_str();
   name = sg->read_str();
   state = sg->read_int();
   plating = sg->read_int();
   speed_capacity = sg->read_int();
   type = sg->read_str();
   cargo_capacity = sg->read_int();
   ammo_capacity = sg->read_int();
   cargo = sg->read_int();
   ammunition = sg->read_int();
   cannons = sg->read_int();
   crew_members = sg->read_int();
   crew_pay = sg->read_int();
   crew_content = sg->read_int();
   crew_optimum = sg->read_int();
   cannon_range = sg->read_int();
   pay_percent = sg->read_int();
   cannon_capacity = sg->read_int();
   plating_capacity = sg->read_int();
   starving = sg->read_bool();
   delay = sg->read_int();
   delay_capacity = sg->read_int();
   food_capacity = sg->read_int();
};


Ship::Ship()
{
};

void Ship::add_food(float quant)
{
   food += quant;
   if (food > food_capacity)
      food = food_capacity;
}


void Ship::recalculate_cargo()
{
   cargo = 0;
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      cargo += gi->get_stock();
}

void Ship::reset_stock()
{
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      gi->set_stock(0);
}

void Ship::set_cannons(int amount)
{
   cannons = amount;
}

int Ship::get_delay()
{
   return delay;
}

// returns true if one's own ship type is superior then the other
bool Ship::has_better_or_equal_shiptype(Ship* ship)
{
   string opp = ship->get_type();

   if (type == "MERCHANT")
      if (opp == "GALLEON" || opp == "BRIG" || opp == "SCHOONER" || opp == "FRIGATE")
         return false;

   if (type == "GALLEON")
      if (opp == "BRIG" || opp == "SCHOONER" || opp == "FRIGATE")
         return false;
   
   if (type == "SCHOONER")
      if (opp == "BRIG" || opp == "FRIGATE")
         return false;

   if (type == "BRIG")
      if (opp == "FRIGATE")
         return false;

   return true;
}

void Ship::save_to_file(Savegame* sg)
{
   sg->writeout(food);
   for (int i=0; i<5; i++)
      goods.at(i).save_to_file(sg);
   sg->writeout(flag);
   sg->writeout(name);
   sg->writeout(state);
   sg->writeout(plating);
   sg->writeout(speed_capacity);
   sg->writeout(type);
   sg->writeout(cargo_capacity);
   sg->writeout(ammo_capacity);
   sg->writeout(cargo);
   sg->writeout(ammunition);
   sg->writeout(cannons);
   sg->writeout(crew_members);
   sg->writeout(crew_pay);
   sg->writeout(crew_content);
   sg->writeout(crew_optimum);
   sg->writeout(cannon_range);
   sg->writeout(pay_percent);
   sg->writeout(cannon_capacity);
   sg->writeout(plating_capacity);
   sg->writeout(starving);
   sg->writeout(delay);
   sg->writeout(delay_capacity);
   sg->writeout(food_capacity);
}

bool Ship::boarding_is_worthwhile(Ship* opponent)
{
   if (opponent->get_type() == "GALLEON")
      return true;
   
   if (opponent->get_type() == "MERCHANT")
      return true;

   if (crew_members <= crew_optimum)
      return false;
   else if (has_better_or_equal_shiptype(opponent))
      return true;
   else if (crew_members > crew_optimum && type != "MERCHANT" && type != "GALLEON")
      return true;

   return false;
}

int Ship::get_stock(string tomatch)
{
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == tomatch)
         return gi->get_stock();

   return 0;
}

bool Ship::has_cannons()
{
   if (cannons > 0)
      return true;
   
   return false;
}

std::string Ship::get_flag()
{
   return flag;
}

void Ship::set_flag(std::string nationflag)
{
   flag = nationflag;
}

void Ship::decrease_ammo()
{
   ammunition -= 1;
   if (ammunition < 0)
      ammunition = 0;
}

string Ship::get_type_descr()
{
   if (type == "SCHOONER")
      return "Schooner";

   if (type == "BRIG")
      return "Brig";

   if (type == "GALLEON")
      return "Galleon";

   if (type == "FRIGATE")
      return "Frigate";

   if (type == "MERCHANT")
      return "Merchant Vessel";

   if (type == "NONE")
      return "None";

   return "";
}

std::vector<Good> Ship::get_goods()
{
   return goods;
}

void Ship::set_name(std::string shipname)
{
   name = shipname;
}

void Ship::set_pay_percent(int percent)
{
   pay_percent = percent;
}

string Ship::get_name()
{
   return name;
}

string Ship::get_type()
{
   return type;
}

void Ship::add_damage()
{
   if (plating == 0)
      state -= gameplay_parameters.get_int("damage_if_plating_0");
   
   if (plating == 1)
      state -= gameplay_parameters.get_int("damage_if_plating_1");

   if (plating == 2)
      state -= gameplay_parameters.get_int("damage_if_plating_2");

   if (plating == 3)
      state -= gameplay_parameters.get_int("damage_if_plating_3");

   if (plating == 4)
      state -= gameplay_parameters.get_int("damage_if_plating_4");

   if (plating == 5)
      state -= gameplay_parameters.get_int("damage_if_plating_5");

   if (plating == 6)
      state -= gameplay_parameters.get_int("damage_if_plating_6");

   if (state < 0)
      state = 0;

   int crewloss = rand() % 100;
   
   // GAMEPLAYFIXME, this formula should be improved

   if (crewloss >= 0 && crewloss < 60)
      crew_members = (int) rint(crew_members *
                          (gameplay_parameters.get_int("crew_losses_in_percent_in_60_percent_of_all_cases")/100));
   
   if (crewloss > 60 && crewloss < 80)
      crew_members = (int) rint(crew_members *
                          (gameplay_parameters.get_int("crew_losses_in_percent_in_20_percent_of_all_cases")/100));

   if (crewloss > 80 && crewloss < 92)
      crew_members = (int) rint(crew_members *
                          (gameplay_parameters.get_int("crew_losses_in_percent_in_12_percent_of_all_cases")/100));

   if (crewloss > 92 && crewloss < 100)
      crew_members = (int) rint(crew_members * 
                          (gameplay_parameters.get_int("crew_losses_in_percent_in_8_percent_of_all_cases")/100));
}

float Ship::get_food_free()
{
   return food_capacity - food;
}

int Ship::get_cannon_capacity()
{
   return cannon_capacity;
}

int Ship::get_food_capacity()
{
   return food_capacity;
}

int Ship::get_plating_capacity()
{
   return plating_capacity;
}

int Ship::get_state()
{
   return state;
}

bool Ship::is_sinking()
{
   if (state <= 0)
      return true;
   
   return false;
}

int Ship::get_plating()
{
   return plating;
}

int Ship::damage()
{
  return plating-=10;
}


int Ship::get_pay_percent()
{
   return pay_percent;
}

float Ship::get_speed()
{
   // GAMEPLAYFIXME, this formula should be improved. It sucks and should grow linearly.
   float speed = speed_capacity * ((float)state/100);

   if (crew_members >= crew_optimum)
      ;
   else if (crew_members >= crew_optimum/2)
      speed = speed * gameplay_parameters.get_float("speed_modifier_if_crew_size_between_50_and_100_percent");
   else
      speed = speed * gameplay_parameters.get_float("speed_modifier_if_crew_size_below_50_percent");

   return speed;
}

int Ship::get_cargo_capacity()
{
   return cargo_capacity;
}

int Ship::get_ammo_capacity()
{
   return ammo_capacity;
}

int Ship::get_cargo()
{
   return cargo;
}

int Ship::get_ammunition()
{
   return ammunition;
}

int Ship::get_cannons()
{
   return cannons;
}

int Ship::get_crew_members()
{
   return crew_members;
}

int Ship::get_crew_pay()
{
   return crew_pay;
}

int Ship::get_crew_content()
{
   return crew_content;
}

int Ship::get_cargo_free()
{
   return (cargo_capacity - cargo);
}


// You need at least half of the optimum of crew members to leave the harbour
bool Ship::enough_crew()
{
   bool ready;
   if (crew_members < (crew_optimum*gameplay_parameters.get_float("how_much_crew_is_needed_for_sailing_away")))
      ready = false;
   else
      ready = true;
   
   return ready;
}

// You need at least 25% ship state to leave the harbour
bool Ship::enough_state()
{
   bool ready;
   if (state < gameplay_parameters.get_int("how_much_ship_state_is_needed_for_sailing_away"))
      ready = false;
   else
      ready = true;
   
   return ready;
}

void Ship::add_good(int amount, string goodid)
{
   for (vector<Good>::iterator gi = goods.begin(); gi != goods.end(); gi++)
      if (gi->get_name() == goodid)
      {
         gi->add_stock(amount);
         cargo += amount;
      }
}

// -1 if a mutiny took place
int Ship::pay_crew(int percent, int currentgold)
{
   Tools t;
   int pay = (pay_percent/100)*crew_pay;

   if (pay > currentgold)
   {
      crew_content -= gameplay_parameters.get_int("decrease_of_crew_satisfaction_if_not_enough_wage_left");
      pay = currentgold;
   }

   int crew_regression = gameplay_parameters.get_int("base_factor_for_crew_satisfaction_regression")
      - (percent/5);
   crew_content -= crew_regression;

   if (crew_content > 100)
      crew_content = 100;
   
   if (crew_content < gameplay_parameters.get_int("low_wage_mutiny_lower_boundary"))
      if (t.rnd_percent(gameplay_parameters.get_int("low_wage_mutiny_probability")))
         pay = -1;

   return pay;
}

void Ship::add_crew_members(int members, int pay)
{
   crew_members += members;
   crew_pay += pay;
}

// returns the price the player has to pay
// -1 if he doesn't have enough dough
int Ship::add_ammunition(int amount, int price, int gold)
{
   int added;

   if (amount <= 0)
      return 0;

   if (amount > (ammo_capacity - ammunition))
      added = (ammo_capacity - ammunition);
   else
      added = amount;

   if ((added*price) <= gold)
      ammunition += added;
   else
      return -1;
   
   return added*price;
}

void Ship::add_abrasion()
{
   state -= (rand() % gameplay_parameters.get_int("maximum_damage_if_abrasion_took_place"));
}

// returns the price the player has to pay
// -1 if he doesn't have enough dough
int Ship::add_cannons(int amount, int price, int gold)
{
   int added;

   if (amount <= 0)
      return 0;

   if (amount > (cannon_capacity - cannons))
      added = (cannon_capacity - cannons);
   else
      added = amount;

   if ((added*price) <= gold)
      cannons += added;
   else
      return -1;
   
   int max_delay = gameplay_parameters.get_int("maximum_cannon_delay");

   if (amount > 0)
      delay = max_delay-((max_delay-delay_capacity)*(cannons/cannon_capacity));

   return added*price;
}

// returns the price the player has to pay
// -1 if he doesn't have enough dough
int Ship::add_plating(int amount, int price, int gold)
{
   int added;

   if (amount <= 0)
      return 0;

   if (amount > (plating_capacity - plating))
      added = (plating_capacity - plating);
   else
      added = amount;

   if ((added*price) <= gold)
      plating += added;
   else
      return -1;
   
   return added*price;
}

void Ship::add_plating(int amount)
{
   if (amount <= 0)
      return;

   if (amount > (plating_capacity - plating))
      plating += (plating_capacity - plating);
   else
      plating += amount;
}

// returns the price the player has to pay
// -1 if he doesn't have enough dough
int Ship::repair(int amount, int price, int gold)
{
   int added;

   if (amount <= 0)
      return 0;

   if (amount > (100 - state))
      added = (100 - state);
   else
      added = amount;

   if ((added*price) <= gold)
      state += added;
   else
      return -1;
   
   return added*price;
}

bool Ship::is_starving()
{
   return starving;
}

void Ship::feed_hungry_mouths()
{
   starving = false;
}

// GAMEPLAYFIXME. The probability of a storm should be configurable as well
void Ship::add_storm_damage()
{
   state -= (rand() % gameplay_parameters.get_int("maximum_damage_if_storm_took_place"));
}

// FIXME, this looks flaky
void Ship::survived_mutiny()
{
   state += 20;
}

void Ship::remain_in_harbour()
{
   crew_content += gameplay_parameters.get_int("improvement_of_crew_mood_when_residing_in_harbour");
   if (crew_content > 100)
      crew_content = 100;
}

float Ship::get_food()
{
   return food;
}

void Ship::consume_food(float consumption)
{
   food -= consumption;
}

void Ship::turn(bool newturn) 
{
   Tools t;

   if (newturn)
   {
      float food_consumption = crew_members * gameplay_parameters.get_float("food_consumption_per_crew_member");

      if (get_food() >= food_consumption)
      {
         consume_food(food_consumption);
         starving = false;
      }
      
      if (get_food() < food_consumption)
      {
         food = 0;
         starving = true;
      }
   }


   if (newturn)
      if (t.rnd_percent(gameplay_parameters.get_int("probability_that_abrasion_takes_place")))
         add_abrasion();
}












