#ifndef SHIP_H
#define SHIP_H

#include "good.h"
#include "savegame.h"
#include "readdata.h"
#include <list>
#include <string>
#include <vector>

class Good;
class Savegame;

class Ship
{
public:
   Ship(std::string shiptype, std::string shipname, int shipstate);
   Ship::Ship(Savegame* sg);
   Ship();

   std::vector<Good> get_goods();
   std::string get_name();
   std::string get_flag();
   std::string get_type();
   std::string get_type_descr();
   int get_stock(std::string tomatch);
   int get_state();
   int get_plating();
   float get_food();
   int damage();
   float get_speed();
   int get_cargo_capacity();
   int get_ammo_capacity();
   int get_cargo_free();
   float get_food_free();
   int get_plating_capacity();
   int get_cannon_capacity();
   int get_cargo();
   int get_ammunition();
   int get_cannons();
   int get_crew_members();
   int get_crew_pay();
   int get_crew_content();
   int get_pay_percent();

   int get_food_capacity();

   void recalculate_cargo();

   void add_food(float quant);
   void feed_hungry_mouths();

   int get_delay();

   void set_name(std::string shipname);
   bool has_better_or_equal_shiptype(Ship* ship);
   bool boarding_is_worthwhile(Ship* ship);

   bool is_sinking();
   bool is_starving();
   bool enough_crew();
   bool enough_state();

   bool has_cannons();

   void set_flag(std::string nationflag);
   void set_pay_percent(int percent);

   void add_good(int amount, std::string goodid);
   void add_crew_members(int members, int pay);

   void reset_stock();

   int add_ammunition(int amount, int price, int gold);
   int add_cannons(int amount, int price, int gold);
   int add_plating(int amount, int price, int gold);
   void add_plating(int amount);
   void survived_mutiny();
   void remain_in_harbour();

   void set_cannons(int amount);
   
   void consume_food(float consumption);

   void add_damage();
   void add_storm_damage();
   void add_abrasion();
   void decrease_ammo();

   int repair(int amount, int price, int gold);

   int pay_crew(int percent, int currentgold);
   void turn(bool newturn);

   void save_to_file(Savegame* sg);

protected:
   readdata gameplay_parameters;

   float food;
   std::vector<Good> goods;
   std::string flag;
   std::string name;
   int state;  // in per cent
   int plating;
   int speed_capacity;
   std::string type;
   int cargo_capacity;
   int ammo_capacity;
   int cargo;
   int ammunition; // deprecated
   int cannons;
   int crew_members;
   int crew_pay;
   int crew_content;
   int crew_optimum;
   int cannon_range;
   int pay_percent;
   int cannon_capacity;
   int plating_capacity;
   bool starving;
   int delay;
   int delay_capacity;
   int food_capacity;
};



#endif








