/*
 *  sprite.cc
 *
 *  Copyright (C) 2004 Christoph Dwertmann
 *
 *  Freebooters is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Freebooters is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with Freebooters; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <SDL.h>
#include <SDL_image.h>
#include <iostream>
#include "replacecol.h"
#include "sprite.h"
#include "tools.h"

using namespace std;

Sprite::Sprite(vector<vector<string> > filename, int x, int y, int w, int h, int a, int m, int r, int g, int b,int tr, int tg, int tb)
{
   Tools t;
   for (vector<vector<string> >:: iterator baum = filename.begin(); baum != filename.end(); baum++) {
      vector<SDL_Surface*> currentbaum;
      for (vector<string>:: iterator element = baum->begin(); element != baum->end(); element++) {
         SDL_Surface* temp = t.load_img(t.sprites_path(*element));
         
         if (temp != 0)
         {
            if (r >= 0 && g >=0 && b >= 0)
               SDL_SetColorKey (temp,SDL_SRCCOLORKEY,SDL_MapRGB(temp->format,r,g,b));
            currentbaum.push_back(temp);
         }
         
      }
      views.push_back(currentbaum);
   }
   rect.x = x;
   rect.y = y;
   if ( w != 0 ) rect.w = w;
   else rect.w = views[0][0]->w;
   if ( h != 0 ) rect.h = h;
   else rect.h = views[0][0]->h;
   
   baumpos=m;
   angle=0;
   rotate(a);
   
}

Sprite::~Sprite()
{
}

void Sprite::setcolor(int r, int g, int b){  
  for (vector<vector<SDL_Surface*> >:: iterator baum = views.begin(); baum != views.end(); baum++) {
    for (vector<SDL_Surface*>:: iterator element = baum->begin(); element != baum->end(); element++) {
      replacered(*element,r,g,b);
    }
  }
}


void Sprite::move(int diff_x, int diff_y)
{
   rect.x += diff_x;
   rect.y += diff_y;
}

void Sprite::move_to(int x, int y)
{
  rect.x = x;
  rect.y = y;
}

void Sprite::rotate_baum(int m)
{
  baumpos=m;
}

void Sprite::rotate(int a)
{
  angle+=a;
  if (angle < 0) angle=360+angle;
  angle=angle%360;
  //  currentview=angle/(360/views[baumpos].size());
  currentview=angle/10;
}

void Sprite::rotate_to(int a)
{
  angle=a;
  angle=angle%360;
  currentview=angle/10;
}

void Sprite::draw(SDL_Surface* target)
{
  if ((Uint32)baumpos > views.size() ) baumpos = views.size()-1;
  image = views[baumpos][currentview];
  SDL_BlitSurface(image, 0, target, &rect);
}

const SDL_Rect& Sprite::get_rect() const
{
   return rect;
}

SDL_Surface* Sprite::get_surface()
{
   return image;
}

void Sprite::unload(){
  for (vector<vector<SDL_Surface*> >:: iterator baum = views.begin(); baum != views.end(); baum++) {
    for (vector<SDL_Surface*>:: iterator element = baum->begin(); element != baum->end(); element++) {
      SDL_FreeSurface(*element);
    }
    baum->clear();
  }
  views.clear();
}







